using Distributed
# # If generating multiple synthetic NoNs at once, you can increase this value to do them in parallel.
addprocs(1)
@everywhere push!(LOAD_PATH, "../")
@everywhere using SyntheticNoNs, LightGraphs, GraphIO
println("done loading packages")

# # types of random graph models at level 1. 
# # it is important to list scale free first. Scale free random graphs are not guaranteed to have the specified number of edges, only an approximation. So, they are generated first, and then other random graph models match their size.
l1rgms = [gen_sf, gen_geo]
l1names = ["sf", "geo"]
l1namestring = join(l1names, "-")
# # sizes of the level 1 networks
l1n = 200
l1m = 800
# # types of random graph models at level 2
l2rgms = [gen_sf, gen_geo]
l2names = ["sf", "geo"]
l2namestring = join(l2names, "-")
# # sizes of the level 2 networks
l2n = 1500
l2m = 10000
# # number of isolated NoNs to generate per level1-level2 combination
l2numgroupsper = 1

# # across-edge and rewire-noise amounts. Multiple can be generated at once.
acrosspercs = [0.05]
rewirepercs = [0.0]


@sync @distributed for rewirepercent in rewirepercs
    @sync @distributed for l2acrosspercent in acrosspercs

        # # location to save the NoN
        basedir = "data/"
        outdir = "$(l1namestring)_$(l1n)_$(l1m)_$(l2namestring)_$(l2n)_$(l2m)_$(l2numgroupsper)_$(l2acrosspercent)_$(rewirepercent)/"
        # # location of the python configuration file that has information about the NoN 
        configf = "scripts/l1l2_config_2mod_$(replace(string(l2acrosspercent), "." => "-"))_$(replace(string(rewirepercent), "." => "-")).py"
        open(configf, "w") do fo
            println(fo, "l1names = $l1names")
            println(fo, "l1namestring = \"-\".join(l1names)")
            println(fo, "l1n = $l1n")
            println(fo, "l1m = $l1m")
            println(fo, "l2names = $l2names")
            println(fo, "l2namestring = \"-\".join(l2names)")
            println(fo, "l2n = $l2n")
            println(fo, "l2m = $l2m")
            println(fo, "l2numgroupsper = $l2numgroupsper")
            println(fo, "l2acrosspercent = $l2acrosspercent")
            println(fo, "rewirepercent = $rewirepercent")
            println(fo, "nondir = f\"data/{l1namestring}_{l1n}_{l1m}_{l2namestring}_{l2n}_{l2m}_{l2numgroupsper}_{l2acrosspercent}_{rewirepercent}/\"")
        end

        # # creation of the NoN using the SyntheticNoNs package.
        non = gen_synthetic_non(l1rgms, l1names, l1n, l1m, l2rgms, l2names, l2n, l2m, l2numgroupsper, l2acrosspercent; rewirepercent=rewirepercent, numprocs=1, seed=0)

        # # formats to save the NoNs in. All three formats are needed, as some programs require certain formats.
        formats = [FlexibleEdgeListFormat("\t", false), FlexibleEdgeListFormat("\t", true), GMLFormat2(1)]
        for format in formats
            save2levelnon("$basedir/$outdir/", non; delim="\t", format=format)
        end
    end
end
