function make_node_to_cutoff_poly(n)
	polycoeffs = [[144.57841751600196, -0.8537443651879738, 0.001196273328253245, -4.428274991676541e-7], [-93.55654369939802, 0.553261844575816, -0.0007824514184745611, 2.92938093398833e-7], [11.784088170890445, -0.06968072120677798, 9.876785706294951e-5, -3.7232820976979834e-8], [-0.5587904651709733, 0.003299689650308382, -2.586357379934949e-6, 1.774431495509709e-9], [0.011985652228627672, -7.067123543331011e-5, 7.645101225958412e-8, -3.8041070254635256e-11], [-0.00011804742467141104, 6.943714644912749e-7, -8.989833415103732e-10, 3.7289092211566464e-13], [4.3405453562545245e-7, -2.5454497896981645e-9, 3.4899261476270964e-12, -1.3611625282065476e-15]]
	cholder = Float64[]
	for c in polycoeffs
		poly = Polynomial(Vector{Float64}(c))
		push!(cholder, poly(n))
	end
	return Polynomial(cholder)
end

function solve_for_cutoff(poly, nedges)
	poly_to_solve = poly - nedges
	r = roots(poly_to_solve)
	realroots = filter(isreal, r)
	return realroots
end

"""
	euclidean_graph_nm()
Can also call this using ```gen_geo``` with the same arguments. Generate a geometric random graph with ```n``` nodes and as close to ```m``` edges as possible. Throws an error when it is not mathematically possible.

### Optional arguments

### Implementation notes
We generated a bunch of ```euclidean_graph```s and then reverse engineered the number of nodes and cutoff needed to reach a target number of edges. Deriving the theoretical expected value given the number of nodes and cutoff would be more optimal.
"""
function euclidean_graph_nm(n::Integer, m::Integer; tries=3, seed=-1, keyargs...)
	coeffpoly = make_node_to_cutoff_poly(n)
	croots = solve_for_cutoff(coeffpoly, m)
	realroots = map(x -> convert(Float64, x), filter(isreal, croots))
	validroots = filter(x -> 100 > x > 0, realroots)
	length(validroots) == 0 && throw(DomainError(m, "not possible to create a euclidean_graph graph with $n nodes and $m edges"))
	cutoff = validroots[1]
	partial_euclidean_graph(x, y; keyargs...) = euclidean_graph_nodist(x, y; cutoff=cutoff, L=100, keyargs...)
	G = try_until_connected(partial_euclidean_graph, tries, (n, 3), seed, keyargs...)
	# euclidean_graph_nodist(n, 3; L=100, cutoff=cutoff, keyargs...)
	return G
end
