import LightGraphs:
	AbstractGraph

import Base:
	eltype,
	isequal,
	==



mutable struct InterEdge
	src::Tuple{Int, Int}
	dst::Tuple{Int, Int}
	is_directed::Bool
end
function InterEdge(src::Tuple{Int, Int}, dst::Tuple{Int, Int}; is_directed=false)
	return InterEdge(src, dst, is_directed)
end
function ==(ie1::InterEdge, ie2::InterEdge)
	# InterEdge((2, 100), (3, 125))
	# InterEdge((3, 125), (2, 100))
	if ie1.is_directed != ie2.is_directed
		return false
	end
	srcind1 = ie1.src[1] # 2
	dstind1 = ie1.dst[1] # 3
	srcnode1 = ie1.src[2] # 100
	dstnode1 = ie1.dst[2] # 125

	srcind2 = ie2.src[1] # 3
	dstind2 = ie2.dst[1] # 2
	srcnode2 = ie2.src[2] # 125
	dstnode2 = ie2.dst[2] # 100

	if ie1.is_directed
		directed_expr = srcind1 == srcind2 && dstind1 == dstind2 && srcnode1 == srcnode2 && dstnode1 == dstnode2
		# println(directed_expr)
		return directed_expr
	else
		undirected_expr = ((srcind1 == srcind2 && dstind1 == dstind2) && (srcnode1 == srcnode2 && dstnode1 == dstnode2)) || ((srcind1 == dstind2 && srcind2 == dstind1) && (srcnode1 == dstnode2 && srcnode2 == dstnode1))
		# println(undirected_expr)
		return undirected_expr
	end

end
zero(::Type{InterEdge}) where T = InterEdge((0, 0), (0, 0))
zero(ie::InterEdge) where T = zero(typeof(ie))


"""

Insert a zero(InterEdge) if you don't want any groups to be connected, but still want the graph of groups to be treated a one graph.
"""
mutable struct GraphOfGroups{GT <: AbstractGraph}
	groups::Vector{GT}
	tags::Vector{String}
	interedges::Vector{InterEdge} # (group, node) to (group, node)
	function GraphOfGroups(groups::Vector{GT}, tags::Vector{String}, interedges::Vector{InterEdge}) where {GT <: AbstractGraph, T <: eltype(GT)}
		length(groups) != length(tags) && throw(DimensionMismatch("Each group must have exactly one tag."))
		# outGG = new{GT}(groups, tags, interedges)
		outGG = new{GT}()
		outGG.groups = groups
		outGG.tags = tags
		outGG.interedges = interedges
		# println(typeof(outGG))
		return outGG
	end
end

function GraphOfGroups(groups::Vector{GT}, tags::Vector{String}) where GT <: AbstractGraph
	outGG = GraphOfGroups(groups, tags, InterEdge[])
	# println(typeof(outGG))
	return outGG
end
function GraphOfGroups(groups::Vector{GT}) where GT <: AbstractGraph
	return GraphOfGroups(groups, repeat([""], length(groups)))
end
function GraphOfGroups()
	return GraphOfGroups([SimpleGraph()], [""])
end
# eltype(GG::GraphOfGroups) = eltype(eltype(GG.groups))
eltype(::Type{GraphOfGroups{GT}}) where GT <: AbstractGraph = GT
eltype(GG::GraphOfGroups{GT}) where GT <: AbstractGraph = eltype(typeof(GG))
# function eltype(GG::GraphOfGroups{GT}) where GT <: AbstractGraph
# 	return eltype(GG.groups)
# 	# return GT
# end
# function eltype(LT::DataType) where LT <: GraphOfGroups{GT}
# 	return GT
# end

function nvg(GG::GraphOfGroups)
	total = 0
	for G in GG.groups
		total += nv(G)
	end
	return total
end


function add_graph!(GG::GraphOfGroups{GT}, G::GT, tag::String) where GT <: AbstractGraph
	push!(GG.groups, G)
	push!(GG.tags, tag)
end

function rem_graph!(GG::GraphOfGroups{GT}, i::Int) where GT <: AbstractGraph
	i in 1:length(GG.groups) || return false
	filter!(x -> !(x.src[1] == i || x.dst[1] == i), GG.interedges)
	for (ieind, ie) in enumerate(GG.interedges)
		# move all the graph indices down one if they come after the one removed
		currentie = ie
		if currentie.src[1] > i
			GG.interedges[ieind] = InterEdge((currentie.src[1]-1, currentie.src[2]), (currentie.dst[1], currentie.dst[2]))
		end
		currentie = GG.interedges[ieind]
		if currentie.dst[1] > i
			GG.interedges[ieind] = InterEdge((currentie.src[1], currentie.src[2]), (currentie.dst[1]-1, currentie.dst[2]))
		end
	end
	deleteat!(GG.groups, i)
	deleteat!(GG.tags, i)
	return true
end

function add_interedge!(GG::GraphOfGroups{GT}, e::InterEdge) where {GT <: AbstractGraph}
	ng = length(GG.groups)
	G1ind = e.src[1]
	G2ind = e.dst[1]
	G1ind != G2ind || return false # intergroup edge can't be between the same group
	G1ind in 1:ng && G2ind in 1:ng || return false # actually between two groups
	G1 = GG.groups[G1ind]
	G2 = GG.groups[G2ind]
	G1nodeind = e.src[2]
	G2nodeind = e.dst[2]
	G1nodeind in 1:nv(G1) && G2nodeind in 1:nv(G2) || return false # nodes actually exist
	push!(GG.interedges, e)
	return true
end

function add_interedge!(GG::GraphOfGroups{GT}, s::Tuple{Int, Int}, d::Tuple{Int, Int}) where GT <: AbstractGraph
	ie = InterEdge(s, d)
	add_interedge!(GG, ie)
end

function rem_interedge!(GG::GraphOfGroups{GT}, e::InterEdge) where {GT <: AbstractGraph}
	ind = findfirst(isequal(e), GG.interedges)
	ind != nothing || return false
	deleteat!(GG.interedges, ind)
	return true
end

function rem_interedge!(GG::GraphOfGroups{GT}, s::Tuple{Int, Int}, d::Tuple{Int, Int}) where GT <: AbstractGraph
	ie = InterEdge(s, d)
	rem_interedge!(GG, ie)
end

function to_simple_graph(GG::GraphOfGroups)
	nodetags = String[]
	startingnodes = [1]
	for i in 2:length(GG.groups)
		push!(startingnodes, startingnodes[i-1] + nv(GG.groups[i-1]))
	end

	combG = SimpleGraph()

	for (Gind, G) in enumerate(GG.groups)
		for (nind, node) in enumerate(vertices(G))
			add_vertex!(combG)
			push!(nodetags, GG.tags[Gind])
		end
		for (eind, edge) in enumerate(edges(G))
			newsrc = src(edge) + startingnodes[Gind] - 1
			newdst = dst(edge) + startingnodes[Gind] - 1
			# println("$newsrc to $newdst")
			add_edge!(combG, newsrc, newdst)
		end
	end

	for (ieind, ie) in enumerate(GG.interedges)
		if ie == zero(ie)
			continue
		end
		G1ind = ie.src[1]
		G1nodeind = ie.src[2]
		G1node = vertices(GG.groups[G1ind])[G1nodeind]
		G2ind = ie.dst[1]
		G2nodeind = ie.dst[2]
		G2node = vertices(GG.groups[G2ind])[G2nodeind]
		add_edge!(combG, startingnodes[G1ind] + findfirst(isequal(G1node), vertices(GG.groups[G1ind])) - 1, startingnodes[G2ind] + findfirst(isequal(G2node), vertices(GG.groups[G2ind])) - 1)
	end

	return combG, nodetags


end

# abstract type AbstractNoNTypes end
# NoNTypes = Union{AbstractGraph, Vector{AbstractGraph}, GraphOfGroups{AbstractGraph}}
NoNType{GT} = GraphOfGroups{GT} where GT <: AbstractGraph
abstract type AbstractNoN{GT <: AbstractGraph, LT <: NoNType} end
# eltype(l::AbstractNoN{LT}) where LT <: NoNType = LT
# abstract type AbstractNoN{GT <: AbstractGraph} end

"""

After creating a NoN, we do not recommend modifying any of the graphs that make it up.
"""
mutable struct NoN{GT <: AbstractGraph, LT <: NoNType{GT}} <: AbstractNoN{GT, LT}
	upper::AbstractNoN{GT, LT}
	self2upper::Dict{Int, Tuple{Int, eltype(GT)}} # ith network at current level maps to (jth network, kth node) at upper level
	self::LT
	lower::AbstractNoN{GT, LT}
	self2lower::Dict{Tuple{Int, eltype(GT)}, Int} # (ith network, jth node) at current level maps to kth network at lower level
	nl::Int
	function NoN(l::LT) where LT <: NoNType
		outL = new{eltype(LT), LT}()
		outL.self = l
		outL.nl = 1
		return outL
	end
end
function NoN()
	GG = GraphOfGroups()
	return NoN(GG)
end
eltype(::Type{NoN{GT, LT}}) where {GT <: AbstractGraph, LT <: NoNType{GT}} = LT
function eltype(l::NoN{GT, LT}) where {GT <: AbstractGraph, LT <: NoNType{GT}}
	# println(typeof(l))
	return eltype(typeof(l))
end


function assign_upper_only!(non, uppernon, self2upper)
	non.upper = uppernon
	non.self2upper = self2upper
end

function assign_lower_only!(non, lowernon, self2lower)
	non.lower = lowernon
	non.self2lower = self2lower
end

function assign_upper!(non, uppernon, self2upper)
	assign_upper_only!(non, uppernon, self2upper)
	upper2self = Dict(value => key for (key, value) in self2upper)
	assign_lower_only!(uppernon, non, upper2self)
end

function assign_lower!(non, lowernon, self2lower)
	assign_lower_only!(non, lowernon, self2lower)
	lower2self = Dict(value => key for (key, value) in self2lower)
	assign_upper_only!(lowernon, non, lower2self)
end
