using Test
using LightGraphs, SyntheticNoNs
using Statistics

@testset "helpers" begin

    arr = [Edge(5, 10), Edge(5, 11), Edge(6, 14), Edge(7, 3), Edge(8, 10)]
    mx = _map_to_sequential!(arr)
    @test arr == [Edge(1, 2), Edge(1, 3), Edge(4, 5), Edge(6, 7), Edge(8, 2)]
    @test mx == 8

end


@testset "Optional params" begin
    @test typeof(gen_sf(200, 800; is_directed=true)) == SimpleDiGraph{Int64}

end


@testset "Randomness of generators" begin

    # G1 = euclidean_graph_nm(200, 800; seed=1)
    # G2 = euclidean_graph_nm(200, 800; seed=1)
    # @test G1 == G2


    # G1 = euclidean_graph_nm(200, 800; seed=4)
    # G2 = euclidean_graph_nm(200, 800; seed=4)
    # @test G1 == G2

    # rgmodels = [erdos_renyi_nm, euclidean_graph_nm, barabasi_albert_nm] #, erdos_renyi_nm]
    rgmodels = [gen_er, gen_geo, gen_sf]
    for model in rgmodels
        for i in 1:10
            G1 = model(200, 800)
            G2 = model(200, 800)
            @test G1 != G2
        end
        for i in 1:10
            # println(i)
            G1 = model(200, 800; seed=i)
            G2 = model(200, 800; seed=i)
            G3 = model(200, 800; seed=i+1)
            G4 = model(200, 800; seed=i-1)
            @test G1 == G2
            @test G1 != G3
            @test G1 != G4
        end
    end



end

@testset "Connectedness of generators" begin
    rgmodels = [gen_er, gen_geo, gen_sf]
    mnames = ["er", "geo", "sf"]
    for (mind, model) in enumerate(rgmodels)
        @testset "$(mnames[mind])" begin
            for i in 1:5
                G = model(200, 800; tries=10)#, seed=i)
                @test length(connected_components(G)) == 1
            end
        end
    end
end

@testset "Size of nm generators" begin

    n = 200
    m = 100000
    @test_throws DomainError(m, "not possible to create a euclidean_graph graph with $n nodes and $m edges because $m > ($n * ($n-1) / 2), which is the maximum number of possible edges.") gen_geo(n, m)

    nnodes = 200:50:1000
    nedges = 800:50:1600
    skip = true
    for i in 1:length(nnodes)
        n = nnodes[i]
        e = nedges[i]
        if skip
            ls = 1:1
        else
            ls = 1:5
        end
        for l in ls
            esizes = []
            for i in 1:5
                G = gen_geo(n, e; seed=i, L=l)
                push!(esizes, length(edges(G)))
            end
            @testset "n $n, m $e, L $l" begin
                # @test min(e-10, 0.95*e) <= mean(esizes) <= max(e+10, 1.05*e)
                @test e <= mean(esizes) <= e
            end
        end

    end


    rgmodels = [gen_er, gen_sf]

    for i in 1:length(nnodes)
        n = nnodes[i]
        e = nedges[i]
        for (mind, model) in enumerate(rgmodels)
            esizes = []
            for i in 1:5
                G = model(n, e; seed=i)
                push!(esizes, length(edges(G)))
            end
            if mind == 1
                @testset "ER" begin
                    @test min(e-10, 0.95*e) <= mean(esizes) <= max(e+10, 1.05*e)
                end
            else
                @testset "SF" begin
                    # println("$n $e $esizes $(mean(esizes))")
                    # println("$(min(e-10, 0.95*e) <= mean(esizes) <= max(e+10, 1.05*e))")
                    @test min(e-10, 0.75*e) <= mean(esizes) <= max(e+10, 1.25*e)
                end
            end
        end
    end



    # for i in 1:5
    #     G = euclidean_graph_nm(200, 800)
    #     push!(esizes, length(edges(G)))
    # end
    # @test min(790, 0.95*800) <= mean(esizes) <= max(810, 1.05*800)

end
