module PSNConstruction
# using Distributed
# addprocs(16)
# using ArgParse
# @everywhere
# push!(LOAD_PATH, "../SyntheticNoNs.jl/src")
# @everywhere
using SyntheticNoNs
# @everywhere
# @everywhere
using DataStructures
# using DataFrames
using LightGraphs
using Distances

export pdbchain2psn, getseqnum, getseqstats, getcoords, getcoordict, writepsn, gen_and_write_psn

# function parsecli()
# 	apsettings = ArgParseSettings()
# 	@add_arg_table apsettings begin
# 		"--basedir", "-b"
# 			help = "base dir"
# 			default = "/scratch/sgu3/extracting-ppi-networks/data/biogrid2pdb/non/"
# 		"--chaindir", "-c"
# 			help = "pdb chain directory"
# 			default = "/scratch/sgu3/extracting-ppi-networks/data/biogrid2pdb/non/pdbchains/"
# 		"--cutoff", "-a"
# 			help = "angstrom cutoff"
# 			arg_type = Float64
# 			default = 6.0
# 		"--atom",
# 			help = "which atom(s) to consider"
# 			default = "all"
# 	end
# 	return parse_args(ARGS, apsettings)
# end
#
# parsedargs = parsecli()
# println(parsedargs)
# u_basedir = parsedargs["basedir"]
# u_chaindir = parsedargs["chaindir"]
# u_cutoff = parsedargs["cutoff"]
# u_atom = parsedargs["atom"]

function getseqnum(line)
	return parse(Int, line[23:26])
end

function getseqstats(pdbf)
	startseq = 0
	stopseq = 0
	open(pdbf, "r") do fi
		lines = split(read(fi, String), "\n")
		fline = lines[1]
		lline = lines[end]
		if lline == ""
			lline = lines[end-1]
		end
		startseq = getseqnum(fline)
		stopseq = getseqnum(lline)
	end
	numnodes = stopseq - startseq + 1
	return startseq, stopseq, numnodes
end

function getcoords(line)
	x = parse(Float64, line[31:38])
	y = parse(Float64, line[39:46])
	z = parse(Float64, line[47:54])
	return [x, y, z]
end

function getcoordict(pdbf)
	coordict = Dict()
	open(pdbf, "r") do fi
		currentnode = 0
		seqs = []
		for line in eachline(fi)
			currentseq = getseqnum(line)
			# currentnode = currentseq - startseq + 1
			coord = getcoords(line)
			# if currentnode in keys(coordict)
			if currentseq in seqs
				push!(coordict[currentnode], coord)
			else
				push!(seqs, currentseq)
				currentnode += 1
				coordict[currentnode] = [coord]
			end
		end
	end
	return coordict
end

function pdbchain2psn(pdbf, cutoff, atom; isweighted=false)
	# startseq, stopseq, numnodes = getseqstats(pdbf)
	coordict = getcoordict(pdbf)
	numnodes = length(keys(coordict))
	G = SimpleGraph(numnodes)

	nodes = collect(keys(coordict))
	for (n1ind, node1) in enumerate(nodes)
		for (n2ind, node2) in enumerate(nodes[n1ind+1:end])
			alldists = []
			for coord1 in coordict[node1]
				for coord2 in coordict[node2]
					dist = euclidean(coord1, coord2)
					if dist <= cutoff
						# println(coord1)
						# println(coord2)
					end
					# could automatically jump to adding edge if dist <= cutoff, but if making weighted networks, need to get all distances so that the weight is the smallest one
					push!(alldists, dist)
				end
			end

			if minimum(alldists) <= cutoff
				add_edge!(G, node1, node2)
			end
		end
	end
	return G
end

function writepsn(outf, G)
	open(outf, "w") do fo
		for edge in edges(G)
			println(fo, src(edge), "\t", dst(edge))
		end
	end
end

function gen_and_write_psn(pdbf, cutoff, atom, outf; isweighted=false)
	G = pdbchain2psn(pdbf, cutoff, atom; isweighted=isweighted)
	writepsn(outf, G)
end



end
# using .PSNConstruction
# using Test
# using LightGraphs
#
# @testset "" begin
# 	pdbf = "ppiscripts/testfiles/dummypdb.txt"
# 	@test getseqnum(readline(pdbf)) == 140
# 	@test getseqstats(pdbf) == (140, 142, 3)
# 	@test getcoords(readline(pdbf)) == [58.293, 29.583, 30.684]
# 	@test getcoordict(pdbf, 140) == Dict{Any,Any}(2 => [[56.881, 26.331, 30.879], [55.829, 25.578, 31.547], [56.176, 25.22, 32.994], [55.29, 25.159, 33.851], [55.524, 24.29, 30.781], [56.733, 23.376, 30.612], [56.336, 21.911, 30.428], [55.359, 21.703, 29.356], [55.5, 22.114, 28.095], [56.59, 22.772, 27.709], [54.54, 21.866, 27.213]],3 => [[57.46, 24.999, 33.271], [57.871, 24.626, 34.619], [57.957, 25.807, 35.574], [57.4, 25.77, 36.653], [59.158, 23.778, 34.608], [58.861, 22.449, 33.888], [59.686, 23.601, 36.028], [59.57, 21.222, 34.414]],1 => [[58.293, 29.583, 30.684], [58.515, 28.109, 30.628], [57.422, 27.413, 31.425], [57.107, 27.829, 32.539], [59.896, 27.723, 31.199], [60.212, 26.211, 31.121], [61.281, 25.746, 32.112], [62.65, 26.12, 31.744], [63.258, 27.251, 32.108], [62.631, 28.149, 32.865], [64.493, 27.508, 31.688]])
# 	testG = pdbchain2psn(pdbf, 6, "all")
# 	expectedG = SimpleGraph(3)
# 	add_edge!(expectedG, 1, 2)
# 	add_edge!(expectedG, 2, 3)
# 	add_edge!(expectedG, 1, 3)
# 	@test testG == expectedG
# 	pdbf2 = "ppiscripts/testfiles/dummypdb-oneatom.txt"
# 	testG = pdbchain2psn(pdbf2, 6, "all")
# 	expectedG = SimpleGraph(5)
# 	add_edge!(expectedG, 1, 2)
# 	add_edge!(expectedG, 1, 3)
# 	add_edge!(expectedG, 2, 3)
# 	add_edge!(expectedG, 3, 4)
# 	add_edge!(expectedG, 4, 5)
# 	add_edge!(expectedG, 1, 5)
# 	@test testG == expectedG
# 	pdbf3 = "ppiscripts/testfiles/dummypdb-twoatom.txt"
# 	testG = pdbchain2psn(pdbf3, 6, "all")
# 	add_edge!(expectedG, 3, 5)
# 	@test testG == expectedG
# end
#
# chaindir = "/scratch/sgu3/extracting-ppi-networks/data/biogrid2pdb/non/pdbchains/"
# cutoff = 6.0
# atom = "all"
# pdbchain2psn("$chaindir/1a02_F.txt", cutoff, atom)
