module ProteinNoNLoader
push!(LOAD_PATH, "./scripts/")
using PSNConstruction
using LightGraphs
# @everywhere
using SyntheticNoNs
using DataStructures

export loadedgelist, gen_protein_non

function readmap(mapf)
    forward = Dict()
    backward = Dict()
    open(mapf, "r") do fi
        for line in eachline(fi)
            larr = split(line)
            id1 = larr[1]
			id1int = tryparse(Int, id1)
            id2 = larr[2]
			id2int = tryparse(Int, id2)
			if id1int != nothing
				id1 = id1int
			end
			if id2int != nothing
				id2 = id2int
			end
            forward[id1] = id2
            backward[id2] = id1
        end
    end
    return forward, backward
end

function loadedgelist(elf)
    nodes = OrderedSet()
    open(elf, "r") do fi
        for line in eachline(fi)
            larr = split(line)
            node1 = parse(Int, larr[1])
            node2 = parse(Int, larr[2])
            push!(nodes, node1)
            push!(nodes, node2)
        end
    end
	if length(nodes) == 0
		return SimpleGraph()
	end
    if minimum(nodes) != 1
        throw(ErrorException("Edgelist probably not in correct format, node ids must start from 1"))
    end
    G = SimpleGraph(maximum(nodes))
    open(elf, "r") do fi
        for line in eachline(fi)
            larr = split(line)
            node1 = parse(Int, larr[1])
            node2 = parse(Int, larr[2])
            add_edge!(G, node1, node2)
        end
    end
    return G
end

function gen_protein_non(ppinumf, psndir, pdb2numf)
    l2G = loadedgelist(ppinumf)
    l2n = nv(l2G)
    l2GG = GraphOfGroups(AbstractGraph[l2G], [""], [InterEdge((0, 0), (0, 0))])

    l2tol1 = Dict()
    pdb2num, num2pdb = readmap(pdb2numf)

    l1Glist = AbstractGraph[]
    for i in 1:l2n
        l2tol1[(1, i)] = i
        if i in keys(num2pdb)
            l1G = loadedgelist("$psndir/$(num2pdb[i]).txt")
            push!(l1Glist, l1G)
        else
            l1G = SimpleGraph()
            push!(l1Glist, l1G)
        end
    end
    l1GG = GraphOfGroups(l1Glist, repeat([""], length(l1Glist)), [InterEdge((0, 0), (0, 0))])
    l1 = NoN(l1GG)
	l2 = NoN(l2GG)
	assign_lower!(l2, l1, l2tol1)
	return l1
end


end
