using Distributed
addprocs(16)
using ArgParse
# using EvalMetrics
using Test
@everywhere using Statistics
@everywhere using HypothesisTests
function parsecli()
	apsettings = ArgParseSettings()
	@add_arg_table apsettings begin
		"--basedir", "-b"
			default = "/scratch/sgu3/cleannondata/"
		"--outdir", "-o"
			default = "/scratch/sgu3/cleannondata/"
		"--goparams", "-g"
			default = "Homo_sapiens-protein-GO.EXP.bp.txt"
		"--classcutoff", "-c"
			arg_type = Int
			default = 20
		"--lr", "-l"
			arg_type = Float64
			default = 0.01
		"--epochs", "-e"
			arg_type = Int
			default = 100
		"--angstromcutoff", "-a"
			arg_type = Float64
			default = 6.0
		"--negclass", "-n"
			default = "negbp"
		"--network"
			default = ""
		"--mode"
			default = ""
	end
	return parse_args(ARGS, apsettings)
end
parsedargs = parsecli()
println(parsedargs)
u_basedir = parsedargs["basedir"]
u_outdir = parsedargs["outdir"]
u_goparams = parsedargs["goparams"]
u_classcutoff = parsedargs["classcutoff"]
u_angstromcutoff = parsedargs["angstromcutoff"]
u_lr = parsedargs["lr"]
u_epochs = parsedargs["epochs"]
u_negclass = parsedargs["negclass"]
u_networktype = ""
u_godir = "$u_outdir/$(u_goparams[1:end-4])-$(u_classcutoff)-$(u_negclass)$(u_networktype)"
u_resoutdir = "$u_outdir/$(u_goparams[1:end-4])-$(u_classcutoff)-$(u_negclass)$(u_networktype)-results/"
u_maskdir = "$u_resoutdir/masks/"
u_seeddir = "$u_resoutdir/indres/"
run(`mkdir -p $u_resoutdir`)
u_mode = parsedargs["mode"]



@everywhere function pred2score(predf)
	scores = Float64[]
	open(predf, "r") do fi
		for line in eachline(fi)
			if occursin("Float", line)
				line = line[9:end-1]
			else
				line = line[2:end-1]
			end
			larr = split(line, ", ")
			score = replace(larr[2], "f" => "e")
			score = parse(Float64, score)
			push!(scores, score)
		end
	end
	return scores
end

@everywhere function readmask(maskf)
	mask = Bool[]
	open(maskf, "r") do fi
		for line in eachline(fi)
			line = split(line)[1]
			if line == "true"
				push!(mask, true)
			else
				push!(mask, false)
			end
		end
	end
	return mask
end

@everywhere function labels2class(labelf)
	classes = Float64[]
	open(labelf, "r") do fi
		for line in eachline(fi)
			larr = split(line)
			if larr[2] == "1"
				push!(classes, 1)
			else
				push!(classes, 0)
			end
		end
	end
	return classes
end

# # get the correct label and the predicted probability of that label
@everywhere function getscoretuples(godir, gof, predf)
	labelf = godir * "/" * gof
	labels = labels2class(labelf)
	gostem = gof[1:end-4]
	scores = pred2score(predf)
	tups = []
	for (lind, label) in enumerate(labels)
		tup = (label, scores[lind])
		push!(tups, tup)
	end
	return tups
end

@everywhere function applymask(arr, mask)
	outarr = []
	for (mind, m) in enumerate(mask)
		if m
			push!(outarr, arr[mind])
		end
	end
	return outarr
end

# # given a k, calculate the measure@k.
@everywhere function calcatk(scoretups, k, measure)
	sortedscores = sort(scoretups, by=x->x[2], rev=true)
	topscores = sortedscores[1:k]
	tps = 0
	for tup in topscores
		if tup[1] == 1.0
			tps += 1
		end
	end
	if measure == "precision"
		return tps / k
	elseif measure == "recall"
		remaining = sortedscores[k+1:end]
		fns = 0
		for r in remaining
			if r[1] == 1.0
				fns += 1
			end
		end
		return tps / (tps + fns)
	elseif measure == "f-score"
		pre = calcatk(scoretups, k, "precision")
		rec = calcatk(scoretups, k, "recall")
		return 2 / (1/pre + 1/rec)
	end
end

@everywhere function calcatks(scoretups, k)
	return calcatk(scoretups, k, "precision"), calcatk(scoretups, k, "recall"), calcatk(scoretups, k, "f-score")
end

# # find k where f-score is highest and precision is greater than recall
@everywhere function findbestk(scoretupcollection)
	karr = []
	for k in 1:length(scoretupcollection[1])
		avgholder = []
		for scoretups in scoretupcollection
			pre, rec, fscore = calcatks(scoretups, k)
			if pre < rec
				push!(avgholder, -1)
			else
				push!(avgholder, fscore)
			end
		end
		avgval = mean(avgholder)
		push!(karr, (k, avgval))
	end
	maxtup = (-1, -1)
	for ktup in karr
		if maxtup[2] <= ktup[2]
			maxtup = ktup
		end
	end
	return maxtup[1]

end

function atkpipeline(godir, resoutdir, maskdir, seeddir, mode, measure; seedrange=1:3)

	lrstr = "0.01"
	if mode == "convs2"
		lrstr = "0.001"
	end

	@sync @distributed for gof in readdir(godir)
		if occursin("sizes", gof)
			continue
		end
		labelf = godir * "/" * gof
		labels = labels2class(labelf)
		gostem = gof[1:end-4]

		actualgostem = ""
		if mode == "convs2"
			actualgostem = replace(gostem, ":" => "_")
		else
			actualgostem = gostem
		end

		scorescontainer = Array{Array{Any, 1}, 1}()
		randomscores = []
		for seed in seedrange
			scoref = seeddir * actualgostem * ".txt_lr$(lrstr)_$(mode)_seed$(seed).results.preds"
			if !isfile(scoref)
				println("$scoref does not exist")
				continue
			end
			scores = getscoretuples(godir, gof, scoref)
			maskf = "$maskdir/$(gostem)_testmask$(seed).txt"
			mask = readmask(maskf)
			maskedscores = applymask(scores, mask)
			maskedlabels = applymask(labels, mask)

			push!(scorescontainer, maskedscores)
			push!(randomscores, sum(maskedlabels) / length(maskedlabels))
		end
		bestk = findbestk(scorescontainer)
		avgholder = []
		for s in scorescontainer
			push!(avgholder, calcatk(s, bestk, measure))
		end

		randomscore = mean(randomscores)
		randomstd = std(randomscores)
		pval = pvalue(OneSampleTTest(mean(avgholder), std(avgholder), 3, randomscore), tail=:right)
		avgf = "$resoutdir/$(gostem)_lr$(lrstr)_$(mode).results.avg.$(measure)"
		open(avgf, "w") do fo
			println(fo, mean(avgholder), " ± ", std(avgholder))
			println(fo, randomscore, " ± ", randomstd)
			println(fo, "pval: $pval")
			println(fo, bestk)
		end

	end
end

measures = ["precision", "recall", "f-score"]
for measure in measures
	atkpipeline(u_godir, u_resoutdir, u_maskdir, u_seeddir, u_mode, measure; seedrange=1:3)
end

