function runorca_ignoreblank(fname; mode="gdv")
    # open(fname, "r") do fi
    #     fline = readline(fi)
    #     farr = split(fline)
    #     if farr[1] == "0" && farr[2] == "0"
    #         return Array{Array{Float64, 1}, 1}()
    #     end
    # end
    if !ispath("$fname.ncount")
        if Sys.iswindows()
            func = "orca.exe"
        else
            func = "orca"
        end
        run(`./scripts/$func 4 $fname $fname.ncount`)
    end
    feats = Array{Array{Float32, 1}, 1}()
    open("$fname.ncount") do fi
        for line in eachline(fi)
            feat = map(x -> parse(Float32, x), split(line))
            push!(feats, feat)
        end
    end
    return feats
end

function non2proc(l1dir, l2dir, procdir, l2n; wantnormalize=true)

	featdict = Dict()

	l2netf = "$l2dir/l2.txt"
	l2feats = runorca_ignoreblank("$l2dir/l2.sz_hdr.txt")
	for (find, feat) in enumerate(l2feats)
		actualfeat = feat
		if wantnormalize
			actualfeat = normalize(feat)
		else
			actualfeat = feat
		end
		featdict[find] = actualfeat
	end
	l2G = loadedgelist(l2netf)
	l1netfs = map(x -> "$l1dir/$(lpad(x, length(string(l2n)), '0')).txt", 1:l2n)
	for (nind, l1netf) in enumerate(l1netfs)
		# nind is also the current level 2 node
		currenttotaln = nv(l2G)

		l1G = loadedgelist(l1netf)
		if nv(l1G) == 0
			continue
		end

		l1feats = runorca_ignoreblank(l1netf[1:end-4] * ".sz_hdr.txt")
		for (find, feat) in enumerate(l1feats)
			actualfeat = feat
			if wantnormalize
				actualfeat = normalize(feat)
			else
				actualfeat = feat
			end
			featdict[find + currenttotaln] = actualfeat
		end

		add_vertices!(l2G, nv(l1G))
		for i in 1:nv(l1G)
			add_edge!(l2G, nind, i + currenttotaln)
		end
		for edge in edges(l1G)
			add_edge!(l2G, src(edge) + currenttotaln, dst(edge) + currenttotaln)
		end
	end

	featarr = []
	open("$procdir/l1l2proc.sz_hdr.txt.ncount", "w") do fo
		for node in 1:nv(l2G)
			push!(featarr, featdict[node])
			println(fo, join(featdict[node], "\t"))
		end
	end

	formats = [FlexibleEdgeListFormat("\t", false), FlexibleEdgeListFormat("\t", true), GMLFormat2(1)]
	savegraph("$procdir/l1l2proc.txt", l2G, formats[1])
	savegraph("$procdir/l1l2proc.sz_hdr.txt", l2G, formats[2])
	return l2G, featarr
end

function non2proc2(l1dir, l2dir, procdir, l2n; wantnormalize=true)

	featdict = Dict()

	l2netf = "$l2dir/l2.txt"
	l2feats = runorca_ignoreblank("$l2dir/l2.sz_hdr.txt")
	for (find, feat) in enumerate(l2feats)
		actualfeat = feat
		if wantnormalize
			actualfeat = normalize(feat)
		else
			actualfeat = feat
		end
		featdict[find] = actualfeat
	end
	l2G = loadedgelist(l2netf)
	l1netfs = map(x -> "$l1dir/$(lpad(x, length(string(l2n)), '0')).txt", 1:l2n)
	for (nind, l1netf) in enumerate(l1netfs)
		# nind is also the current level 2 node
		currenttotaln = nv(l2G)

		l1G = loadedgelist(l1netf)
		if nv(l1G) == 0
			continue
		end

		l1feats = runorca_ignoreblank(l1netf[1:end-4] * ".sz_hdr.txt")
		avgfeats = sum(l1feats, dims=1)[1] ./ length(l1feats)
		featdict[1 + currenttotaln] = normalize(avgfeats)
		# for (find, feat) in enumerate(l1feats)
		# 	actualfeat = feat
		# 	if wantnormalize
		# 		actualfeat = normalize(feat)
		# 	else
		# 		actualfeat = feat
		# 	end
		# 	featdict[find + currenttotaln] = actualfeat
		# end

		add_vertices!(l2G, 1)
		add_edge!(l2G, nind, 1 + currenttotaln)
		# for i in 1:nv(l1G)
		# 	add_edge!(l2G, nind, i + currenttotaln)
		# end
		# for edge in edges(l1G)
		# 	add_edge!(l2G, src(edge) + currenttotaln, dst(edge) + currenttotaln)
		# end
	end

	featarr = []
	open("$procdir/l1l2proc2.sz_hdr.txt.ncount", "w") do fo
		for node in 1:nv(l2G)
			push!(featarr, featdict[node])
			println(fo, join(featdict[node], "\t"))
		end
	end

	formats = [FlexibleEdgeListFormat("\t", false), FlexibleEdgeListFormat("\t", true), GMLFormat2(1)]
	savegraph("$procdir/l1l2proc2.txt", l2G, formats[1])
	savegraph("$procdir/l1l2proc2.sz_hdr.txt", l2G, formats[2])
	return l2G, featarr
end



function loadfeatarrs(featarrf)
	featarr = Vector{Vector{Float32}}()
	open(featarrf, "r") do fi
		for line in eachline(fi)
			feat = split(line)
			feat = map(x -> parse(Float32, x), feat)
			push!(featarr, feat)
		end
	end
	return featarr
end

function makeproc(l1dir, l2dir, procdir, l2n)
	if isfile("$procdir/l1l2proc.txt")
		println("already made non")
		return loadedgelist("$procdir/l1l2proc.txt"), loadfeatarrs("$procdir/l1l2proc.sz_hdr.txt.ncount")
	else
		println("making non")
		G, featarr = non2proc(l1dir, l2dir, procdir, l2n)
		println("made")
		return G, featarr
	end
end

function makeproc2(l1dir, l2dir, procdir, l2n)
	if isfile("$procdir/l1l2proc2.txt")
		println("already made non")
		return loadedgelist("$procdir/l1l2proc2.txt"), loadfeatarrs("$procdir/l1l2proc2.sz_hdr.txt.ncount")
	else
		println("making non")
		G, featarr = non2proc2(l1dir, l2dir, procdir, l2n)
		println("made")
		return G, featarr
	end
end


function getprocnodes(procf)
	nnodes = 0
	open(procf, "r") do fi
		fline = readline(fi)
		nnodes = parse(Int, split(fline)[1])
	end
	return nnodes
end

function remakelabels(godir, newgodir, procf)
	if ispath(newgodir)
		println("already remade the labels")
		return
	end
	if Sys.iswindows()
		if !ispath(newgodir)
			run(`powershell mkdir $newgodir`)
		end
	else
		run(`mkdir -p $newgodir`)
	end
	nnodes = getprocnodes(procf)
	for godata in readdir(godir)
		gof = "$godir/$godata"
		currentline = 1
		open("$newgodir/$godata", "w") do fo
			open(gof, "r") do fi
				for line in eachline(fi)
					println(fo, line)
					currentline += 1
				end
			end
			for ind in currentline:nnodes
				println(fo, 0, "\t", 0)
			end
		end
	end
end

function writemask2(mask, outf; numlines=18708)
	linestowrite = 0
	if numlines < length(mask)
		linestowrite = numlines
	else
		linestowrite = length(mask)
	end
    open(outf, "w") do fo
        for i in 1:linestowrite
            write(fo, string(mask[i]), "\n")
        end
    end
end


using Test
@testset "nontest" begin

	# testdir = "ppiscripts/testfiles/proctest/"
	# G, featarr = non2proc("$testdir/l1/", "$testdir/l2/", "$testdir", 5)
	# expectedG = SimpleGraph(17)
	# add_edge!(expectedG, 1, 2)
	# add_edge!(expectedG, 2, 3)
	# add_edge!(expectedG, 3, 4)
	# add_edge!(expectedG, 4, 5)
	# add_edge!(expectedG, 2, 5)

	# add_edge!(expectedG, 1, 6)
	# add_edge!(expectedG, 1, 7)
	# add_edge!(expectedG, 1, 8)
	# add_edge!(expectedG, 6, 7)
	# add_edge!(expectedG, 7, 8)
	# add_edge!(expectedG, 6, 8)

	# add_edge!(expectedG, 2, 9)
	# add_edge!(expectedG, 2, 10)
	# add_edge!(expectedG, 9, 10)

	# add_edge!(expectedG, 3, 11)
	# add_edge!(expectedG, 3, 12)
	# add_edge!(expectedG, 3, 13)
	# add_edge!(expectedG, 11, 12)
	# add_edge!(expectedG, 12, 13)

	# add_edge!(expectedG, 5, 14)
	# add_edge!(expectedG, 5, 15)
	# add_edge!(expectedG, 5, 16)
	# add_edge!(expectedG, 5, 17)
	# add_edge!(expectedG, 14, 15)
	# add_edge!(expectedG, 15, 16)
	# add_edge!(expectedG, 16, 17)
	# @test G == expectedG
	# @test map(x -> round(x, digits=4), featarr[3]) == map(x -> round(x, digits=4), Float32[0.48507125007266594, 0.7276068751089989, 0.24253562503633297, 0.0, 0.0, 0.24253562503633297, 0.24253562503633297, 0.0, 0.24253562503633297, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0])
	# @test map(x -> round(x, digits=4), featarr[12]) == map(x -> round(x, digits=4), Float32[0.8944271909999159, 0.0, 0.4472135954999579, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0])

	# @test G == loadedgelist("ppiscripts/testfiles/proctest/l1l2proc.txt")

	# expectedfeatarr = loadfeatarrs("ppiscripts/testfiles/proctest/l1l2proc.sz_hdr.txt.ncount")
	# for (find, feat) in enumerate(featarr)
	# 	@test map(x -> round(x, digits=4), feat) == map(x -> round(x, digits=4), expectedfeatarr[find])
	# end
end
