using ArgParse
using EvalMetrics
using Test
using Statistics
using HypothesisTests
function parsecli()
	apsettings = ArgParseSettings()
	@add_arg_table apsettings begin
		"--basedir", "-b"
			default = "/scratch/sgu3/cleannondata/"
		"--outdir", "-o"
			default = "/scratch/sgu3/cleannondata/"
		"--goparams", "-g"
			default = "Homo_sapiens-protein-GO.EXP.bp.txt"
		"--classcutoff", "-c"
			arg_type = Int
			default = 20
		"--lr", "-l"
			arg_type = Float64
			default = 0.01
		"--epochs", "-e"
			arg_type = Int
			default = 100
		"--negclass", "-n"
			default = "negbp"
		"--network"
			default = ""
		"--mode"
			default = ""
	end
	return parse_args(ARGS, apsettings)
end
parsedargs = parsecli()
println(parsedargs)
u_basedir = parsedargs["basedir"]
u_outdir = parsedargs["outdir"]
u_goparams = parsedargs["goparams"]
u_classcutoff = parsedargs["classcutoff"]
u_lr = parsedargs["lr"]
u_epochs = parsedargs["epochs"]
u_negclass = parsedargs["negclass"]
u_networktype = ""
u_godir = "$u_outdir/$(u_goparams[1:end-4])-$(u_classcutoff)-$(u_negclass)$(u_networktype)"
u_resoutdir = "$u_outdir/$(u_goparams[1:end-4])-$(u_classcutoff)-$(u_negclass)$(u_networktype)-results/"
u_maskdir = "$u_resoutdir/masks/"
u_seeddir = "$u_resoutdir/indres/"
u_mode = parsedargs["mode"]
run(`mkdir -p $u_resoutdir`)


# # get probability that each protein is predicted to have the given GO term
function pred2score(predf)
	scores = Float64[]
	open(predf, "r") do fi
		for line in eachline(fi)
			if occursin("Float", line)
				line = line[9:end-1]
			else
				line = line[2:end-1]
			end
			larr = split(line, ", ")
			score = replace(larr[2], "f" => "e")
			score = parse(Float64, score)
			push!(scores, score)
		end
	end
	return scores
end

# # read mask to know what data is in training/val/testing
function readmask(maskf)
	mask = Bool[]
	open(maskf, "r") do fi
		for line in eachline(fi)
			line = split(line)[1]
			if line == "true"
				push!(mask, true)
			else
				push!(mask, false)
			end
		end
	end
	return mask
end


function labels2class(labelf)
	classes = Float64[]
	open(labelf, "r") do fi
		for line in eachline(fi)
			larr = split(line)
			if larr[2] == "1"
				push!(classes, 1)
			else
				push!(classes, 0)
			end
		end
	end
	return classes
end


# # godir: directory containing files corresponding to GO terms. For each GO term, we indicate if a protein is annotated by that GO term or not.
# # resoutdir: output directory
# # maskdir: directory where data masks are, i.e., training/val/testing split
# # seeddir: directory of where the prediction probabilities are
# # mode: feature
function auprpipeline(godir, resoutdir, maskdir, seeddir, mode; seedrange=1:3)
	for gof in readdir(godir)
		if occursin("sizes", gof)
			continue
		end
		labelf = godir * "/" * gof
		labels = labels2class(labelf)
		gostem = gof[1:end-4]

		actualgostem = ""
		if mode == "convs2"
			actualgostem = replace(gostem, ":" => "_")
		else
			actualgostem = gostem
		end

		auprvals = Float64[]
		expectedscores = Float64[]
		lrstr = "0.01"
		if mode == "convs2"
			lrstr = "0.001"
		end
		for seed in seedrange
			scoref = seeddir * actualgostem * ".txt_lr$(lrstr)_$(mode)_seed$(seed).results.preds"
			if !isfile(scoref)

				println("$scoref does not exist")
				continue
			end


			# # preparing data for julia's aupr computation using the EvalMetrics.jl package
			scores = pred2score(scoref)

			maskf = "$maskdir/$(gostem)_testmask$(seed).txt"
			mask = readmask(maskf)

			maskedlabels = Float64[]
			maskedscores = Float64[]
			for (mind, m) in enumerate(mask)
				if m
					push!(maskedlabels, labels[mind])
					push!(maskedscores, scores[mind])
				else

				end
			end
			# # calculating AUPR
			auprval = au_prcurve(maskedlabels, maskedscores)
			resultf = "$(scoref[1:end-6]).aupr"
			open(resultf, "w") do fo
				println(fo, auprval)
				println(fo, sum(maskedlabels) / length(maskedlabels))
			end
			push!(auprvals, auprval)
			push!(expectedscores, sum(maskedlabels) / length(maskedlabels))
		end

		# # statistical testing to see if the aupr value is greater than random
		avgf = "$resoutdir/$(gostem)_lr$(lrstr)_$(mode).results.avg.aupr"
		pval = pvalue(OneSampleTTest(mean(auprvals), std(auprvals), 3, mean(expectedscores)), tail=:right)
		open(avgf, "w") do fo
			println(fo, mean(auprvals), " ± ", std(auprvals))
			println(fo, mean(expectedscores), " ± ", std(expectedscores))
			println(fo, "pval: $pval")
		end

	end
end

auprpipeline(u_godir, u_resoutdir, u_maskdir, u_seeddir, u_mode; seedrange=1:3)

