using ArgParse
using Distributed
addprocs(16)
# using EvalMetrics
using Test
@everywhere using Statistics
@everywhere using StatsBase
@everywhere using HypothesisTests
@everywhere using MultipleTesting
function parsecli()
	apsettings = ArgParseSettings()
	@add_arg_table apsettings begin
		"--basedir", "-b"
			default = "/scratch/sgu3/cleannondata/"
		"--outdir", "-o"
			default = "/scratch/sgu3/cleannondata/"
		"--goparams", "-g"
			default = "Homo_sapiens-protein-GO.EXP.bp.txt"
		"--classcutoff", "-c"
			arg_type = Int
			default = 20
		"--lr", "-l"
			arg_type = Float64
			default = 0.01
		"--epochs", "-e"
			arg_type = Int
			default = 100
		"--angstromcutoff", "-a"
			arg_type = Float64
			default = 6.0
		"--negclass", "-n"
			default = "negbp"
		"--network"
			default = ""

	end
	return parse_args(ARGS, apsettings)
end
parsedargs = parsecli()
println(parsedargs)
u_basedir = parsedargs["basedir"]
u_outdir = parsedargs["outdir"]
u_goparams = parsedargs["goparams"]
u_classcutoff = parsedargs["classcutoff"]
u_angstromcutoff = parsedargs["angstromcutoff"]
u_lr = parsedargs["lr"]
u_epochs = parsedargs["epochs"]
u_negclass = parsedargs["negclass"]
u_networktype = ""
u_godir = "$u_outdir/$(u_goparams[1:end-4])-$(u_classcutoff)-$(u_negclass)$(u_networktype)"
u_resoutdir = "$u_outdir/$(u_goparams[1:end-4])-$(u_classcutoff)-$(u_negclass)$(u_networktype)-results/"
u_maskdir = "$u_resoutdir/masks/"
u_seeddir = "$u_resoutdir/indres/"
run(`mkdir -p $u_resoutdir`)

# # read the uncorrected p-value
function getuncorrectedpval(resultf)
	pval = 0
	open(resultf, "r") do fi
		readline(fi)
		readline(fi)
		pval = parse(Float64, split(readline(fi))[2])
	end
	return pval
end

# # perform correction
# # resdir: directory of results
# # methods: list containing approaches to correct over
# # golist: list of GO terms
# # measure: measure we are correcting for
function correctpvals(resdir, methods, golist, measure)

	# # getting p-values of each approach
	pvals = Float64[]
	for method in methods
		lrstr = "0.01"
		if method == "convs2"
			lrstr = "0.001"
		end
		for goterm in golist
			resultsf = "$(resdir)/$(goterm)_lr$(lrstr)_$(method).results.avg.$(measure)"
			pval = getuncorrectedpval(resultsf)
			push!(pvals, pval)
		end
	end

	# # doing two types of correction, we ultimately use Benjamini-Hochberg (FDR)
	bfpvals = adjust(pvals, Bonferroni())
	bhpvals = adjust(pvals, BenjaminiHochberg())
	currenti = 1
	for method in methods
		lrstr = "0.01"
		if method == "convs2"
			lrstr = "0.001"
		end

		# # write the corrected p-value for each GO term to file
		for goterm in golist
			resultsf = "$(resdir)/$(goterm)_lr$(lrstr)_$(method).results.avg.$(measure)"
			bfresultsf = "$(resdir)/$(goterm)_lr$(lrstr)_$(method).results.avg.bf.$(measure)"
			bhresultsf = "$(resdir)/$(goterm)_lr$(lrstr)_$(method).results.avg.bh.$(measure)"


			open(resultsf, "r") do fi
				line1 = readline(fi)
				line2 = readline(fi)
				open(bfresultsf, "w") do fo1
					open(bhresultsf, "w") do fo2
						println(fo1, line1)
						println(fo2, line1)
						println(fo1, line2)
						println(fo2, line2)
						println(fo1, "bfpval: $(bfpvals[currenti])")
						println(fo2, "bhpval: $(bhpvals[currenti])")
					end
				end
			end

			currenti += 1
		end
	end

end

u_gof = "/scratch/sgu3/cleannondata/Homo_sapiens-protein-GO.EXP.bp-20-negbp/sizes.sorted.txt"
function getgolist(gof)
	golist = []
	open(gof, "r") do fi
		for line in eachline(fi)
			larr = split(line)
			term = larr[1]
			size = parse(Int, larr[2])
			if size >= 20
				push!(golist, term)
			end
		end
	end
	return golist
end
u_golist = getgolist(u_gof)
@test length(u_golist) == 131


# # for a given measure, we aim to compare different approaches to each other. This means correction needs to be done over all approaches for that measure. However, in this example, since we only run l2gdv, we only have l2gdv here.
u_methods = ["l2gdv"]
u_measures = ["aupr", "precision", "recall", "f-score"]

for measure in u_measures
	correctpvals(u_resoutdir, u_methods, u_golist, measure)
end
