import sys;
import importlib
import tensorflow as tf;
tf.compat.v1.disable_eager_execution()
from tensorflow.keras.callbacks import EarlyStopping
from tensorflow.keras.layers import Input, Dropout, Dense
from tensorflow.keras.models import Model
from tensorflow.keras.optimizers import Adam
from tensorflow.keras.regularizers import l2
from sklearn.preprocessing import Normalizer
from non_gcn import StandardGCN, l2NoNGCN, l1NoNGCN;
import newloader
import socket;
import numpy as np;
from scipy import stats

import glob;
import multiprocessing as mp;
from functools import partial
import networkx as netx;
import os;
import gc;
import subprocess;
import time;

# # where data is
baseoutd = "data/"

non_configf = sys.argv[1]
nonp = importlib.import_module(non_configf);

gl_epochs = int(sys.argv[2])
gl_mode = sys.argv[3]

wantl1informative = False;

gl_numprocs = 32

# # reading NoN parameters from the config file
gl_l2n = nonp.l2n
gl_l2m = nonp.l2m
gl_l2names = nonp.l2names
gl_l1n = nonp.l1n
gl_l1m = nonp.l1m
gl_l1names = nonp.l1names
gl_numlabs = len(gl_l1names) * len(gl_l2names)
gl_l2numnodestotal = gl_l2n
gl_l2numgroupsofeach = nonp.l2numgroupsper
gl_l2numgroupstotal = gl_l2numgroupsofeach * gl_numlabs
gl_l2numnodespergroup = int(gl_l2n / gl_l2numgroupstotal)
gl_l2numedges = gl_l2m
gl_l2numedgesacross = int(nonp.l2acrosspercent * gl_l2m)
gl_l2numedgeswithin = int((gl_l2m - gl_l2numedgesacross) / gl_l2numgroupstotal)
gl_nondir = nonp.nondir

gl_savedmodels = sys.argv[4]
if gl_savedmodels == "x":
	gl_wantload = False
else:
	gl_wantload = True

# # loading NoN from files
l1As, l2A, labs = newloader.loadnon(gl_nondir, gl_l2n, numprocs=gl_numprocs)


# # helper function to compute GDVs of nodes
def run_orca(fname):
	# # if file of nodes' GDVs does not exist, run orca to compute
	if not os.path.isfile(fname + ".ncount"):
		subprocess.call(["./scripts/orca", "4", fname, fname + ".ncount"]);

	# # open the file containing nodes' GDVs and read them as a "matrix"
	# # the ith row of the matrix corresponds to the GDV of the ith node
	featvecs = [];
	with open(fname + ".ncount", "r") as fi:
		nodenum = 0
		for line in fi:
			larr = list(map(int, line.split()));
			featvecs.append(larr);
	return np.array(featvecs);


# # mode: approach you want to use
# # l2n: number of nodes in level 2 network (number of entities)
# # labels: one-hot encoding of labels. ith element corresponds to label of ith node
# # learning_rate: self explanatory
# # outname: stem of output file names
# # randseed: random seed. So that the random sampling is reproducible
# # numprocs: number of processes to use
# # normalize: True to normalize graphlet features
# # inwantload: True to load an existing model
def run_log_reg_on_feats(mode, l2n, labels, learning_rate, outname, randseed, numprocs, normalize=False, inwantload=False):

	# # auxilary code for naming files later
	if normalize:
		normalizestring = "_norm"
	else:
		normalizestring = ""

	noisestr = ""

	print("mode", mode);

	tf.keras.backend.clear_session()

	nclasses = gl_numlabs

	# # example uses one epoch, change this value for number of epochs to train
	epochs = gl_epochs
	# # train in one batch for fairness with GCNs, as GCNs cannot do batched training
	batch_size = l2n

	resultsoutname = outname + "_" + "lr" + str(learning_rate) + "_" + mode + "_seed" + str(randseed) + normalizestring + noisestr + ".txt";

	# # L1 GDVM feature of networks
	if mode == "l1gdvm":
		# # compute GDVMs of level 1 networks in parallel using python "multiprocessing" library, determined by $numprocs
		filenames = list(map(lambda x: outname + "l1/l1_" + noisestr + str(x).zfill(len(str(l2n))) + ".sz_hdr.txt", range(1, l2n+1)));
		pool = mp.Pool(numprocs);
		l1_feats = np.array(pool.map(run_orca, filenames));
		pool.close()
		pool.join()
		# # convert GDVM into a feature vector of a network
		final_feats = np.reshape(l1_feats, (l2n, len(l1_feats[0]) * 15));
		print("@@@ finished counting graphlets GDVM for every l1 network @@@")
	# # L2 GDV feature of nodes
	elif mode == "l2gdv":
		l2_filename = outname + "/l2/l2.txt"
		l2_feats = run_orca(l2_filename[0:-4] + ".sz_hdr.txt");
		final_feats = l2_feats;
		print("@@@ finished counting graphlets for l2 network @@@")
	# # L1 GDVM + L2 GDV feature of NoN entities
	elif mode == "l1l2gdvmgdvconcat":
		# # L1 GDVM feature of level 1 networks
		filenames = list(map(lambda x: outname + "l1/l1_" + noisestr + str(x).zfill(len(str(l2n))) + ".sz_hdr.txt", range(1, l2n+1)));
		pool = mp.Pool(numprocs);
		l1_feats = np.array(pool.map(run_orca, filenames));
		pool.close()
		pool.join()
		l1_feats = np.reshape(l1_feats, (l2n, len(l1_feats[0]) * 15));
		# # L2 GDV feature of level 2 nodes
		l2_filename = outname + "/l2/l2.txt"
		l2_feats = run_orca(l2_filename[0:-4] + ".sz_hdr.txt");
		# # concatenation
		final_feats = np.concatenate((l1_feats, l2_feats), axis=1);
	else:
		print("Mode does not exist.")
		sys.exit();

	# # normalize graphlet counts, like done for the NormGraphlets in Fazle E. Faisal, et al. (2017), GRAFENE, Nature Scientific Reports. 
	if normalize:
		data_normalizer = Normalizer(norm="l2").fit(final_feats);
		final_feats = data_normalizer.transform(final_feats);

	# # instead of breaking the data into training, validation, and testing sets, we mask out the data. For example, when training the classifier using train_mask, only those data instances in train_mask affect the loss function.
	# # mask generation depends on the random seed $randseed. Masks generated for different features but the same seed are the same, allowing for fair comparison between different features.
	train_mask, val_mask, test_mask = newloader.generate_masks_nnodes(l2n, labels.shape[1], 0.1, randseed);
	print("@@@ masks computed @@@")

	# # dimension of features
	fs = np.repeat([np.array(final_feats[0].shape)], l2n, axis=0);

	# # logistic regression model
	feats_in = Input(shape=tuple(fs[0],))
	output = Dense(nclasses,
				  activation='softmax',
				  use_bias=False)(feats_in)

	# # not loading a trained model
	if not inwantload:
		print("training model, not loading it")

		# # tensorflow setup
		modemodel = Model(inputs=[feats_in], outputs=output)
		optimizer = Adam(lr=learning_rate)

		modemodel.compile(optimizer=optimizer,
			loss='categorical_crossentropy',
			weighted_metrics=['acc', tf.keras.metrics.AUC(curve="ROC"), tf.keras.metrics.AUC(curve="PR") ])
		modemodel.summary()

		modemodel.fit([final_feats],
			labels,
			sample_weight=train_mask,
			epochs=epochs,
			batch_size=batch_size,
			shuffle=False,  # Shuffling data means shuffling the whole graph
			)
		print('Evaluating ' + mode + ' model on validation.')
		eval_results = modemodel.evaluate([final_feats],
		labels,
		sample_weight=val_mask,
		batch_size=batch_size)
		
		# # use the model to make predictions
		predictions = modemodel.predict([final_feats], batch_size=batch_size)

		print('Done.\n'
			'loss: {}\n'
			'accuracy: {}\n'
			'AUROC: {}\n'
			'AUPR: {}\n'.format(*eval_results))


		# # write the masks, prediction probabilities, and results to file
		np.savetxt(resultsoutname[0:-4] + "_trmask.txt", train_mask);
		np.savetxt(resultsoutname[0:-4] + "_valmask.txt", val_mask);
		np.savetxt(resultsoutname[0:-4] + "_temask.txt", test_mask);
		np.savetxt(resultsoutname[0:-4] + "_probs.txt", predictions);
		with open(resultsoutname[0:-4] + ".results", "w") as fo:
			fo.write("learning_rate " + str(learning_rate) + "\n");
			fo.write("epochs " + str(epochs) + "\n");
			fo.write("batch_size " + str(batch_size) + "\n");
			fo.write('loss: {}\n'
					 'accuracy: {}\n'
					 'AUROC: {}\n'
					 'AUPR: {}\n'.format(*eval_results));
		
		# # save the model so that once hyperparameters are determined, the best model can be loaded
		modemodel.save(resultsoutname[0:-4] + ".savedmodel")

	# # loading a trained model
	else:
		print("loading model")
		savedmodelf = ""
		with open(gl_savedmodels, "r") as fi:
			for line in fi:
				larr = line.split()[0]
				targetmode = larr.split("_")[-3]
				if mode == targetmode:
					savedmodelf = larr
					break

		savedmodelf = savedmodelf.replace("seed0", f"seed{randseed}")
		print(savedmodelf);

		# # use loaded model on the test data
		newmodel = tf.keras.models.load_model(savedmodelf);
		neweval_results = newmodel.evaluate([final_feats],
			labels,
			sample_weight=test_mask,
			batch_size=batch_size)

		# # save results on the test data
		with open(savedmodelf[0:-11] + ".test.results", "w") as fo:
			fo.write("inlearning_rate " + str(learning_rate) + "\n");
			fo.write("epochs " + str(epochs) + "\n");
			fo.write("batch_size " + str(batch_size) + "\n");
			fo.write('loss: {}\n'
					 'accuracy: {}\n'
					 'AUROC: {}\n'
					 'AUPR: {}\n'.format(*neweval_results));

lrs = [0.1]
for lr in lrs:
	for i in range(3):
		# # number of processes to use when computing features of level 1 networks, etc.
		npr = 32
		run_log_reg_on_feats(gl_mode, gl_l2numnodestotal, labs, lr, gl_nondir, i, npr, normalize=True, inwantload=gl_wantload);


