import importlib
import tensorflow as tf;
tf.compat.v1.disable_eager_execution()
import sys;
from tensorflow.keras.callbacks import EarlyStopping
from tensorflow.keras.layers import Input, Dropout, Dense
from tensorflow.keras.models import Model
from tensorflow.keras.optimizers import Adam
from tensorflow.keras.regularizers import l2
from sklearn.preprocessing import Normalizer
from non_gcn import StandardGCN, l2NoNGCN, l1NoNGCN;
import newloader
import numpy as np;
from scipy import sparse;

import glob;
import multiprocessing as mp;
from functools import partial
import networkx as netx;
import time;
import copy;
import subprocess;
import gc;
import os;
import socket

non_configf = sys.argv[1];
nonp = importlib.import_module(non_configf);
wantl1informative = False

sysinmode = sys.argv[2]
sysinchansize = sys.argv[3]
sysinseed = sys.argv[4]
gl_epochs = int(sys.argv[5])
gl_bestmodels = sys.argv[6]

numprocs = 32

# # read in NoN information from the config file
l2n = nonp.l2n
l2m = nonp.l2m

l1n = nonp.l1n
l1m = nonp.l1m

l1names = nonp.l1names
l2names = nonp.l2names

numlabs = len(l1names) * len(l2names)
l2numnodestotal = l2n
l2numgroupsofeach = nonp.l2numgroupsper
l2numgroupstotal = l2numgroupsofeach * numlabs
l2numnodespergroup = int(l2n / l2numgroupstotal)
l2numedges = l2m

l2numedgesacross = int(nonp.l2acrosspercent * l2m)
l2numedgeswithin = int((l2m - l2numedgesacross) / l2numgroupstotal)

outd = nonp.nondir
fnamestem = ""


# # same function as in non_log_reg.py
def run_orca(fname):

	if not os.path.isfile(fname + ".ncount"):
		subprocess.call(["./scripts/orca", "4", fname, fname + ".ncount"]);

	featvecs = [];
	with open(fname + ".ncount", "r") as fi:
		nodenum = 0
		for line in fi:
			larr = list(map(int, line.split()));
			featvecs.append(np.array(larr));
	return np.array(featvecs);

# # inl1rgms: unused, as synthetic NoNs are generated outside of this function now
# # inl1n: number of nodes in level 1 networks
# # inl1m: number of edges in level 1 networks
# # inl2rgms: unused, as synthetic NoNs are generated outside of this function now
# # inl2numnodestotal: number of nodes in level 2 networks
# # inl2numgroupsofeach: unused, number of isolated NoNs generated per label
# # inl2numnodespergroup: unused, from a previous way of generating NoNs
# # inl2numedgeswithin: unused, from a previous way of generating NoNs
# # inl2numedgesacross: unused
# # inoutd: output directory
# # infnamestem: stem of output files
# # infeatsize: feature size
# # inchannels: size of hidden layers
# # indropout: how much dropout to use inbetween GCN layers
# # inlearning_rate: learning rate
# # mode: type of initial features
# # inoutname: similar to infnamestem
# # inrandseed: random seed
# # innumprocs: number of processes to use
# # innormalize: whether to normalize features or not
# # inwanttoloadmodel: whether to load model or not
def run_non_gcn(inl1rgms, inl1n, inl1m, inl2rgms, inl2numnodestotal, inl2numgroupsofeach, inl2numnodespergroup, inl2numedgeswithin, inl2numedgesacross, inoutd, infnamestem, infeatsize, inchannels, indropout, inlearning_rate, mode, inoutname, inrandseed, innumprocs, innormalize=False, inwanttoloadmodel=False):

	noisestr = ""

	print("running in mode", mode)

	tf.keras.backend.clear_session()

	if innormalize:
		innormalizestring = "_norm"
	else:
		innormalizestring = ""

	# # load the NoN from file
	l1_As, l2_A, labels = newloader.loadnon(inoutd, inl2numnodestotal, numprocs=numprocs)

	# # dimensions of data
	l2n = l2_A.shape[0];
	inl1n = l1_As[0].shape[0];

	# # hyperparameters

	nclasses = numlabs
	reg_rate = 5e-4 / 2
	numconvs = "convs2"
	epochs = gl_epochs
	batch_size = inl2numnodestotal

	# # featsize used if random initial features are used
	featsizesinfunc = [infeatsize];
	# # dimensions of hidden layer
	channelssinfunc = [inchannels];

	for featsize in featsizesinfunc:
		for channels in channelssinfunc:

			print("@@@@ for featsize and channel and indropout and lr and seed", featsize, channels, indropout, inlearning_rate, inrandseed);

			if mode == "random":
				np.random.seed(0)
				l2_X = np.random.sample(size=(l2n, featsize));
				l1_Xs = np.random.sample(size=(l2n, inl1n, featsize));


			elif mode == "graphlets":
				# # get graphlet initial features
				l2_X = run_orca(inoutname + "/l2/l2.sz_hdr.txt")
				filenames = list(map(lambda x: inoutname + f"/l1/l1_" + str(x).zfill(len(str(l2n))) + ".sz_hdr.txt", range(1, l2n+1)));
				pool = mp.Pool(innumprocs);
				l1_Xs_list = pool.map(run_orca, filenames);
				pool.close()
				pool.join()

				# # format data into what tensorflow wants
				featsize = l2_X.shape[1];
				l1_Xs = np.zeros((l2n, inl1n, featsize))
				for i in range(l2n):
					if innormalize:
						data_normalizer = Normalizer(norm="l2").fit(l1_Xs_list[i]);
						temp = data_normalizer.transform(l1_Xs_list[i])
						l1_Xs[i] = copy.deepcopy(temp)
						del temp
					else:

						l1_Xs[i] = copy.deepcopy(l1_Xs_list[i]);

				if innormalize:
					data_normalizer = Normalizer(norm="l2").fit(l2_X);
					l2_X = data_normalizer.transform(l2_X);
				del l1_Xs_list;

				print("@@@ finished making graphlet features @@@")
			else:
				print("mode does not exist")
				sys.exit()


			# # data dimensions for tensorflow model
			l2_n = l2n
			l1_ns = [l1_As[0].shape]
			print(l1_ns)
			l2_f = l2_X.shape[1];
			l1_fs = [l1_Xs[0].shape]

			print("@@@ finished getting the shapes of the various matrices @@@")


			# # GCN transformation of adjacency matrices, see Kipf and Welling (2016)
			l2_fltr = l2NoNGCN.preprocess(l2_A).astype("f4");
			print("finished preproccing l2_fltr")
			l1_fltrs = []
			for i in range(0, len(l1_As)):
				l1_fltrs.append(l1NoNGCN.preprocess(l1_As[i]).astype("f4"))
			l1_fltrs = np.array(l1_fltrs)


			print("finished preproccing l1_fltrs")
			del l2_A
			del l1_As

			print("@@@ finished preprocessing @@@")


			for newrandseed in range(3):
				# # generate same masks as logistic regression since the same starting random seed is used, can fairly compare trainig/val/testiing splits
				train_mask, val_mask, test_mask = newloader.generate_masks_nnodes(l2n, labels.shape[1], 0.1, newrandseed);
				print("@@@ generated the masks @@@")

				if not inwanttoloadmodel:
					print("training the model, not loading it")

					# # making the GNN model in tensorflow
					l2_X_in = Input(shape=(l2_f, ))
					l1_X_in = Input(shape=tuple(l1_fs[0],))

					l2_fltr_in = Input(shape=(None, ))
					l1_fltr_in = Input(shape=tuple(l1_ns[0], ))

					l2_indropout_1 = Dropout(indropout)(l2_X_in)
					l1_indropout_1 = Dropout(indropout)(l1_X_in)

					l2_graph_conv_1 = l2NoNGCN(channels,
					                       activation='relu',
					                       kernel_regularizer=l2(reg_rate),
					                       use_bias=False)([l2_indropout_1, l1_indropout_1, l2_fltr_in, l1_fltr_in])
					l1_graph_conv_1 = l1NoNGCN(channels,
					                        activation='relu',
					                       kernel_regularizer=l2(reg_rate),
					                       use_bias=False)([l2_indropout_1, l1_indropout_1, l2_fltr_in, l1_fltr_in])

					l2_indropout_2 = Dropout(indropout)(l2_graph_conv_1)
					l1_indropout_2 = Dropout(indropout)(l1_graph_conv_1)

					l2_graph_conv_2 = l2NoNGCN(channels,
					                       activation='relu',
					                       use_bias=False)([l2_indropout_2, l1_indropout_2, l2_fltr_in, l1_fltr_in])
					l1_graph_conv_2 = l1NoNGCN(channels,
					                       activation='relu',
					                       use_bias=False)([l2_indropout_2, l1_indropout_2, l2_fltr_in, l1_fltr_in])

					# # 3 GCN layers
					# l2_indropout_3 = Dropout(indropout)(l2_graph_conv_2)
					# l1_indropout_3 = Dropout(indropout)(l1_graph_conv_2)

					# l2_graph_conv_3 = l2NoNGCN(channels,
					#                        activation='relu',
					#                        use_bias=False)([l2_indropout_3, l1_indropout_3, l2_fltr_in, l1_fltr_in])
					# l1_graph_conv_3 = l1NoNGCN(channels,
					#                        activation='relu',
					#                        use_bias=False)([l2_indropout_3, l1_indropout_3, l2_fltr_in, l1_fltr_in])

					output = Dense(nclasses,
					      activation='softmax',
					      use_bias=False)(l2_graph_conv_2)

					# # build model
					nonmodel = Model(inputs=[l2_X_in, l1_X_in, l2_fltr_in, l1_fltr_in], outputs=output)
					optimizer = Adam(lr=inlearning_rate)

					nonmodel.compile(optimizer=optimizer,
					            loss='categorical_crossentropy',
					            weighted_metrics=['acc', tf.keras.metrics.AUC(curve="ROC"), tf.keras.metrics.AUC(curve="PR") ])
					nonmodel.summary()


					print(l2_X.shape)
					print(l2_fltr.shape)
					print(l1_Xs.shape)
					print(l1_fltrs.shape)
					print(labels.shape)
					print(train_mask.shape)
					print(batch_size)
					print(l1_fltrs.shape)


					# # train model, only use training data in the loss function, with train_mask
					nonmodel.fit([l2_X, l1_Xs, l2_fltr, l1_fltrs],
					        labels,
					        sample_weight=train_mask,
					        epochs=epochs,
					        batch_size=batch_size,
					        shuffle=False,  # Shuffling data means shuffling the whole graph

					        )

					print('Evaluating NoN model on validation.')
					# # only evaluate on validation data, with val_mask
					eval_results = nonmodel.evaluate([l2_X, l1_Xs, l2_fltr, l1_fltrs],
					                            labels,
					                            sample_weight=val_mask,
					                            batch_size=batch_size)


					# # only predict on testing data, with test_mask
					predictions = nonmodel.predict([l2_X, l1_Xs, l2_fltr, l1_fltrs], batch_size=batch_size)


					print('Done.\n'
					    'Val loss: {}\n'
					    'Val accuracy: {}'.format(*eval_results))

							

					# # save various information about the model to file
					resultsinoutname = inoutname + "_featsize" + str(featsize) + "_channels" + str(channels) + "_epochs" + str(epochs) + "_dropout" + str(indropout) + "_lr" + str(inlearning_rate) + "_mode" + mode + "_seed" + str(newrandseed) + innormalizestring + noisestr + "_convs2.txt";


					nonmodel.save(resultsinoutname[0:-4] + ".savedmodel")

					np.savetxt(resultsinoutname[0:-4] + "_nongcn_trmask.txt", train_mask);
					np.savetxt(resultsinoutname[0:-4] + "_nongcn_valmask.txt", val_mask);
					np.savetxt(resultsinoutname[0:-4] + "_nongcn_temask.txt", test_mask);
					np.savetxt(resultsinoutname[0:-4] + "_probs.txt", predictions);
					with open(resultsinoutname[0:-4] + ".results", "w") as fo:
						fo.write("featsize " + str(featsize) + "\n");
						fo.write("channels " + str(channels) + "\n");
						fo.write("indropout " + str(indropout) + "\n");
						fo.write("reg_rate " + str(reg_rate) + "\n");
						fo.write("inlearning_rate " + str(inlearning_rate) + "\n");
						fo.write("epochs " + str(epochs) + "\n");
						fo.write("batch_size " + str(batch_size) + "\n");
						fo.write('loss: {}\n'
					    		 'accuracy: {}\n'
					    		 'AUROC: {}\n'
					    		 'AUPR: {}\n'.format(*eval_results));
						
					
				else:

					print("loading model")
					savedmodelf = "";
					with open(gl_bestmodels, "r") as fi:
						for line in fi:
							larr = line.split()[0]
							if numconvs in larr:
								savedmodelf = larr
								break

					savedmodelf = savedmodelf.replace("seed0", f"seed{newrandseed}")

					# savedmodelf = inoutname + savedmodelf.split("/")[-1].split("seed")[0] + "seed" + str(newrandseed) + savedmodelf.split("/")[-1].split("seed")[1][1:]

					newmodel = tf.keras.models.load_model(savedmodelf);
					neweval_results = newmodel.evaluate([l2_X, l1_Xs, l2_fltr, l1_fltrs],
					                            labels,
					                            sample_weight=test_mask,
					                            batch_size=batch_size)
					print('Done.\n'
					    'Test loss: {}\n'
					    'Test accuracy: {}'.format(*neweval_results))
					print(savedmodelf[0:-11] + ".test.results")
					with open(savedmodelf[0:-11] + ".test.results", "w") as fo:
						fo.write("featsize " + str(featsize) + "\n");
						fo.write("channels " + str(channels) + "\n");
						fo.write("indropout " + str(indropout) + "\n");
						fo.write("reg_rate " + str(reg_rate) + "\n");
						fo.write("inlearning_rate " + str(inlearning_rate) + "\n");
						fo.write("epochs " + str(epochs) + "\n");
						fo.write("batch_size " + str(batch_size) + "\n");
						fo.write('Test loss: {}\n'
					    		 'Test accuracy: {}\n'
					    		 'Test AUROC: {}\n'
					    		 'Test AUPR: {}\n'.format(*neweval_results));


noisestr = ""


featsizes = [256] # unused, only relevant for random
channelsizes = [int(sysinchansize)]
dropouts = [0.0];
lrs = [0.01];
modes = [sysinmode]

if gl_bestmodels == "x":
	wanttoloadmodel = False
else:
	wanttoloadmodel = True


# # loop over different hyperparameters
for mode in modes:
	for featsize in featsizes:
		for channels in channelsizes:
			for learning_rate in lrs:
				for dropout in dropouts:
					for randomseed in [int(sysinseed)]:
						if mode == "graphlets":
							wanttonormalize = True;
						else:
							wanttonormalize = False;

						if wanttonormalize:
							normalizestring = "_norm"
						else:
							normalizestring = ""

						run_non_gcn([], l1n, l1m, [], l2numnodestotal, l2numgroupsofeach, l2numnodespergroup, l2numedgeswithin, l2numedgesacross, outd, fnamestem, featsize, channels, dropout, learning_rate, mode, inoutname=outd, inrandseed=randomseed, innumprocs=32, innormalize=wanttonormalize, inwanttoloadmodel=wanttoloadmodel);


