using Distributed
if Sys.iswindows()
    MAXPROCS = 4
else
    MAXPROCS = 4
    println("temp maxprocs 4")
end
addprocs(MAXPROCS)
@everywhere using StatsBase
using Random
using Test

function readyml(yname)
    return YAML.load_file(yname)
end

function gen_and_save_non_from_yml(yname)
    nonparams = readyml(yname)
    return gen_and_save_non_from_params(nonparams)
end

function gen_and_save_non_from_params(nonparams)
    nondir = ""
    idstring = ""
    if Sys.iswindows()
        idstring = nonparams["idstring"]
        nondir = "E:\\nondata\\$(idstring)/"
        nonparams["nondir"] = nondir
    else
        idstring = nonparams["idstring"]
        nondir = nonparams["nondir"]
    end

    non = NoN()
    if isfile("$nondir/l2/l2.sz_hdr.txt") && isfile("$nondir/l1/$(nonparams["l2n"]).sz_hdr.txt")
        println("NoN already exists.")
    else
        println("Generating and saving NoN.")
        non = gen_synthetic_non(nonparams)
        formats = [FlexibleEdgeListFormat("\t", false), FlexibleEdgeListFormat("\t", true), GMLFormat2(1)]
        for format in formats
            save2levelnon("$nondir", non; delim="\t", format=format)
        end
    end
    return non
end

function loadlabels(labelf)
    labels = Array{Array{Float64, 1}, 1}()
    open(labelf) do fi
        for line in eachline(fi)
            label = map(x -> parse(Float64, x), split(line))
            push!(labels, label)
        end
    end
    return labels
end

@everywhere function runorca(fname; mode="gdv")
    # open(fname, "r") do fi
    #     fline = readline(fi)
    #     farr = split(fline)
    #     if farr[1] == "0" && farr[2] == "0"
    #         return Array{Array{Float64, 1}, 1}()
    #     end
    # end
    if !ispath("$fname.ncount")
        if Sys.iswindows()
            func = "orca.exe"
        else
            func = "orca"
        end
        run(`./scripts/$func 4 $fname $fname.ncount`)
    end
    feats = Array{Array{Float64, 1}, 1}()
    open("$fname.ncount") do fi
        for line in eachline(fi)
            feat = map(x -> parse(Float64, x), split(line))
            push!(feats, feat)
        end
    end
    if length(feats) == 0
        push!(feats, repeat(Float64[0], 15))
    end
    return feats
end

@everywhere function getgcm(feats, outf)
    relfeats = feats[:, [1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12]]
    if !isfile(outf)
        ρmat = corspearman(relfeats)

        if any(isnan, ρmat)
            feats = repeat(Float64[0], 121)
            open(outf, "w") do fo
                for ele in feats
                    println(fo, ele)
                end
            end
            # println(typeof(feats), " if is nan")
            return feats
        else
            ρvec = reduce(hcat, ρmat)
            feats = []
            open(outf, "w") do fo
                for ele in ρvec
                    println(fo, ele)
                    push!(feats, ele)
                end
            end
            # println(typeof(feats), " if is spearman")
            return feats
        end

    else
        outfeats = Float64[]
        open(outf, "r") do fi
            for line in eachline(fi)
                val = parse(Float64, split(line)[1])
                push!(outfeats, val)
            end
        end
        if length(outfeats) == 0
            # println(typeof(feats), " file is empty")
            return repeat(Float64[0], 121)
        end
        # println(typeof(outfeats), " if from file")
        return outfeats
    end
end

@everywhere function runpgd(fname)
    linestat = read(`wc -l $fname`, String)
    numlines = parse(Int, split(linestat)[1])
    if numlines == 0
        return repeat([0.0], 17)
    end
    if !ispath("$fname.macro")
        # open("$fname.mtx", "w") do fo
        #     open("$fname", "r") do fi
        #         header = readline(fi)
        #         n = split(header)[1]
        #         newheader = string(n, "\t", header)
        #         println(fo, newheader)
        #         for line in eachline(fi)
        #             println(fo, line)
        #         end
        #     end
        # end
        read(`./scripts/pgd -f $fname --macro $fname.macro`)
    end
    # read(`rm $fname.mtx`)
    feat = Float64[]
    open("$fname.macro", "r") do fi
        for line in eachline(fi)
            larr = split(line)
            count = parse(Float64, larr[3])
            push!(feat, count)
        end
    end
    return feat
end

@everywhere function run_node2vec_l1(fname)
    linestat = read(`wc -l $fname`, String)
    numlines = parse(Int, split(linestat)[1])
    if numlines == 0
        return repeat([0.0], 128)
    end
    if !isfile("$fname.node2vec")
        read(`./scripts/node2vec -i:$fname -o:$fname.node2vec`)
    end
    featdict = Dict()
    featvecs = Array{Array{Float64, 1}, 1}()
    open("$fname.node2vec", "r") do fi
        readline(fi)
        for line in eachline(fi)
            larr = split(line)
            nodeid = parse(Int, larr[1])
            vec = map(x -> parse(Float64, x), larr[2:end])
            push!(featvecs, vec)
        end
    end
    featvecs = mean(featvecs, dims=1)[1]
    return featvecs
end

function run_node2vec_l2(fname, l2n)
    if !isfile("$fname.node2vec")
        read(`./scripts/node2vec -i:$fname -o:$fname.node2vec`)
    end
    featdict = Dict()
    featvecs = Array{Array{Float64, 1}, 1}()
    nodeset = Set()
    open("$fname.node2vec", "r") do fi
        readline(fi)
        for line in eachline(fi)
            larr = split(line)
            nodeid = parse(Int, larr[1])
            push!(nodeset, nodeid)
            vec = map(x -> parse(Float64, x), larr[2:end])
            featdict[nodeid] = vec
            # push!(featvecs, vec)
        end
    end
    for i in 1:l2n
        if !(i in nodeset)

            push!(featvecs, repeat([0.0], 128))
        else
            push!(featvecs, featdict[i])
        end
    end
    return featvecs
end

function parl1pgd(l1dir, pool, l2n)
    files = map(x -> "$l1dir/$(lpad(x, length(string(l2n)), "0")).txt", 1:l2n)
    featarr = pmap(runpgd, pool, files)
    return featarr
end

function loadl1orca(l1dir, pool, l2n)
    files = map(x -> "$l1dir/$(lpad(x, length(string(l2n)), "0")).sz_hdr.txt", 1:l2n)
    featsarr = pmap(runorca, pool, files)
    feats = pmap(x -> foldr((y, z) -> vcat(y, z), x), pool, featsarr)
    return feats
end

function loadl1gcm(l1dir, pool, l2n)
    files = map(x -> "$l1dir/$(lpad(x, length(string(l2n)), "0")).sz_hdr.txt", 1:l2n)
    feats = pmap(runorca, pool, files)
    # println(typeof(feats), " after pmap runorca")
    partialreduce(x) = reduce(hcat, x)
    feats = pmap(partialreduce, feats)
    # println(typeof(feats), " after partial reduce")
    feats = pmap(transpose, feats)
    # println(typeof(feats), " after transpose")
    outfiles = map(x -> "$l1dir/$(lpad(x, length(string(l2n)), "0")).gcm.txt", 1:l2n)
    feats = Array{Array{N, 1} where N <: Number, 1}(pmap(getgcm, pool, feats, outfiles))
    # println(typeof(feats), " after pmap getgcm")
    return feats
end

function loadl1node2vec(l1dir, pool, l2n)
    files = map(x -> "$l1dir/$(lpad(x, length(string(l2n)), "0")).txt", 1:l2n)
    feats = Array{Array{N, 1} where N <: Number, 1}(pmap(run_node2vec_l1, pool, files))
    return feats
end




function loadfeats(nondir, l2n, mode; numprocs=1, wantnormalize=true, goterm="")
    pool = WorkerPool(collect(1:numprocs))
    l1dir = "$(nondir)/l1/"
    l2dir = "$(nondir)/l2/"
    feats = Array{Array{Float64, 1}, 1}()
    wantnormalize = true
    if mode == "l1gdvm"
        feats = loadl1orca(l1dir, pool, l2n)
    elseif mode == "l1gfv"
        feats = parl1pgd(l1dir, pool, l2n)
    elseif mode == "l1gcdm"
        feats = loadl1gcm(l1dir, pool, l2n)
        println(typeof(feats), " after l1gcdm")
        println(size(feats), " after l1gcdm")
    elseif mode == "l2gdv"
        feats = runorca("$l2dir/l2.sz_hdr.txt")
        # println(typeof(feats), " after l2gdv")
        # println(size(feats), " after l2gdv")
    elseif mode == "l1gfvl2gdv"
        l1feats = parl1pgd(l1dir, pool, l2n)
        l2feats = runorca("$l2dir/l2.sz_hdr.txt")
        feats = vcat.(l1feats, l2feats)
    elseif mode == "l1gdvml2gdv"
        l1feats = loadl1orca(l1dir, pool, l2n)
        l2feats = runorca("$l2dir/l2.sz_hdr.txt")
        feats = vcat.(l1feats, l2feats)
    elseif mode == "l1l2gcdmgdvconcat_norm"
        l1feats = loadl1gcm(l1dir, pool, l2n)
        l2feats = runorca("$l2dir/l2.sz_hdr.txt")
        l2feats = normalize!.(l2feats)
        # println(l2feats[274])
        # l1feats = map(x -> sum(x) != 0 ? normalize!(x) : x, l1feats)
        feats = vcat.(l1feats, l2feats)
        # println(feats[274])
        # pool = WorkerPool(collect(1:1))
        # testfeats = loadl1gcmtest("/scratch/sgu3/non-embedding/proteindata/l1/", pool, 2)
        # l2feats = runorca("/scratch/sgu3/non-embedding/proteindata/l2/l2.sz_hdr.txt")
        # using LinearAlgebra
        # l2feats = normalize!.(l2feats)
        # feats = vcat.(testfeats, l2feats[1:2])
        # println(feats)
    elseif mode == "l1node2vec"
        pool = WorkerPool(collect(1:4))
        feats = loadl1node2vec(l1dir, pool, l2n)
        wantnormalize = false
    elseif mode == "l2node2vec"
        feats = run_node2vec_l2("$l2dir/l2.txt", l2n)
        wantnormalize = false
    elseif mode == "l1l2node2vecconcat"
        l1feats = loadl1node2vec(l1dir, pool, l2n)
        l2feats = run_node2vec_l2("$l2dir/l2.txt", l2n)
        feats = vcat.(l1feats, l2feats)
        wantnormalize = false
    elseif mode == "l2l2graphletsn2v"
        l2feats = runorca("$l2dir/l2.sz_hdr.txt")
        l2feats = normalize!.(l2feats)
        n2vfeats = run_node2vec_l2("$l2dir/l2.txt", l2n)
        feats = vcat.(l2feats, n2vfeats)
    elseif mode == "l1l2l2gcdmgdvn2v"
        l1feats = loadl1gcm(l1dir, pool, l2n)
        l2feats = runorca("$l2dir/l2.sz_hdr.txt")
        l2feats = normalize!.(l2feats)
        n2vfeats = run_node2vec_l2("$l2dir/l2.txt", l2n)
        feats = vcat.(l1feats, l2feats)
        feats = vcat.(feats, n2vfeats)
    elseif mode == "random"
        feats = [rand(15) for i in 1:l2n]
    else
        throw(ArgumentError("Not a valid feature type."))
    end

    # featset = Set()
    # for feat in feats
        # push!(featset, typeof(feat))
    # end
    # println(featset)
    if wantnormalize
        return map(x -> sum(x) != 0 ? normalize!(x) : x, feats)
    else
        return feats
    end



end




function loadmask(maskf; nnodes=15000)
    maskarr = ones(Bool, nnodes, 1)
    nodecount = 1
    open(maskf, "r") do fi
        for line in eachline(fi)
            larr = split(line)
            val = parse(Float64, larr[1])
            val = convert(Bool, val)
            maskarr[nodecount] = val
            nodecount += 1
        end
    end
    # return reshape(maskarr, length(maskarr), 1)
    return maskarr
end

function makestartingmask(labs)
    mastermask = ones(Bool, 1, length(labs))
    for (lind, lab) in enumerate(labs)
        if sum(lab) == 0
            mastermask[lind] = 0
        end
    end
    return mastermask
end

function getlabelclassarray(labels)
    labelsarr = []
    for i in 1:length(labels[1])
        relevantinds = []
        for (lind, label) in enumerate(labels)
            if label[i] == 1
                push!(relevantinds, lind)
            end
        end
        push!(labelsarr, relevantinds)
    end
    return labelsarr
end

function stratified_mask_sampling(completelabels, completemask, p; seed=-1)
    if p > 1
        throw(ErrorException("Can't remove more elements than the length of the array!"))
    end
    if seed > -1
        Random.seed!(seed)
    end
    updatedlabels = deepcopy(completelabels)
    labelsarr = getlabelclassarray(completelabels)

    labelcounts = map(x -> length(x), labelsarr)
    # labelpercs = labelcounts ./ sum(labelcounts)
    numstochoose = labelcounts .* p
    # println(numstochoose)
    trainmask = deepcopy(completemask)
    testmask = zeros(Bool, 1, length(completemask))
    for i in 1:length(completelabels[1])
        counter = 0
        while counter < numstochoose[i]
            randind = rand(1:length(labelsarr[i]))
            classind = labelsarr[i][randind]
            if trainmask[classind] == 0
                continue
            end
            trainmask[classind] = 0
            updatedlabels[classind] = [0, 0]
            testmask[classind] = 1
            deleteat!(labelsarr[i], randind)
            counter += 1
        end
    end
    return trainmask, testmask, updatedlabels
end

# function gen_train_val_test_masks(feats::Array{Array{T, 1}, 1}, labs::Array{Array{T, 1}, 1}, p::Float64; seed=-1) where T <: Number
function gen_train_val_test_masks(feats, labs, p; seed=-1)
    if p > 1
        throw(ErrorException("Can't remove more elements than the length of the array!"))
    end
    if seed > -1
        Random.seed!(seed)
    end
    mastermask = makestartingmask(labs)

    trainmask, testmask, updatedlabels = stratified_mask_sampling(labs, mastermask, p; seed=seed+rand(1:1000000))
    trainmask, valmask, updatedlabels = stratified_mask_sampling(updatedlabels, trainmask, p/(1-p); seed=seed+rand(1:1000000))
    return trainmask, valmask, testmask
end

function writemask(mask, outf)
    open(outf, "w") do fo
        for i in 1:length(mask)
            write(fo, string(mask[i]), "\n")
        end
    end
end

if Sys.iswindows()

else
    println("tests off at the moment")
    #
    @testset "masks" begin
    #     # testf = "ppiscripts/testfiles/testlabs.txt"
    #     # labels = loadlabels(testf)
    #     # trainmask, valmask, testmask = gen_train_val_test_masks(labels, labels, 0.1; seed=0)
    #     # @test trainmask == Bool[1 0 0 0 1 0 0 0 1 1]
    #     # @test valmask ==   Bool[0 0 0 0 0 1 0 0 0 0]
    #     # @test testmask ==  Bool[0 1 0 0 0 0 0 0 0 0]
    #
    #     masktestf = "ppiscripts/testfiles/dummylabs-masktest.txt"
    #     labels = loadlabels(masktestf)
    #     startmask = makestartingmask(labels)
    #     @test startmask == Bool[0 0 1 1 1 0 1 1 0 1 1 0 1 1 0]
    #     labelsarr = getlabelclassarray(labels)
    #     @test labelsarr == [Int[3, 4, 7, 8, 10, 11], Int[5, 13, 14]]
    #
    #     Random.seed!(0)
    #     trainmask, testmask, updatedlabels = stratified_mask_sampling(labels, startmask, 1/3; seed=0+rand(1:1000000))
    #     @test trainmask == Bool[0 0 1 0 1 0 1 1 0 0 1 0 0 1 0]
    #     @test testmask == Bool[0 0 0 1 0 0 0 0 0 1 0 0 1 0 0]
    #     trainmask, valmask, updatedlabels2 = stratified_mask_sampling(updatedlabels, trainmask, 1/2; seed=0+rand(1:1000000))
    #     @test trainmask == Bool[0 0 0 0 1 0 1 0 0 0 1 0 0 0 0]
    #     @test valmask == Bool[0 0 1 0 0 0 0 1 0 0 0 0 0 1 0]
    #     @test sum(trainmask .⊻ valmask .⊻ testmask) == 9
    #
    #     fulltrmask, fullvalmask, fulltemask = gen_train_val_test_masks(labels, labels, 1/3; seed=0)
    #     @test fulltrmask == trainmask
    #     @test fullvalmask == valmask
    #     @test fulltemask == testmask
    #     # trainmask, valmask, testmask = gen_train_val_test_masks(labels, labels, 1/3; seed=0)
    #     # @test trainmask == Bool[0 0 1 1 0 0 1 1 0 1 1 0 0 0 0]
    #     # @test testmask ==  Bool[0 0 0 0 0 0 0 0 0 0 0 0 1 1 0]
    #     # @test valmask ==   Bool[0 0 0 0 1 0 0 0 0 0 0 0 0 0 0]
    end

    #
    @testset "spearman" begin
    #     netf = "/scratch/sgu3/non-embedding/proteindata/l1/l1_00274.sz_hdr.txt"
    #     feats = runorca(netf)
    #     feats = transpose(reduce(hcat, feats))
    #     feats = getgcm(feats, "/scratch/sgu3/non-embedding/proteindata/l1/l1_00274.gcm.txt")
    #     # println(typeof(feats))
    #     @test reshape(feats, (1, length(feats))) == reduce(hcat, Float64[1.0 0.8968437140190963 0.9940438494300771 0.834871701241656 0.9731149562273284 0.7614668093037148 0.9784513347398944 0.8801006621130096 0.8177765581200408 0.9266545747580092 0.9950881060448081; 0.8968437140190963 0.9999999999999998 0.895102101857471 0.9710884820611638 0.9636005845871033 0.9534950355801798 0.8888971129999148 0.8957249145383875 0.977535983673108 0.9760078798048083 0.8901094274029715; 0.9940438494300771 0.895102101857471 1.0 0.8331490206409989 0.9779912568555786 0.761734786836928 0.9902476288270063 0.8872862085681309 0.8227310530509145 0.9079905454268854 0.9982814527400847; 0.834871701241656 0.9710884820611638 0.8331490206409989 1.0 0.9149408445530416 0.947585881710825 0.8288713693959477 0.8557068486892699 0.9630447457742378 0.9418333683726063 0.8277230901218361; 0.9731149562273284 0.9636005845871033 0.9779912568555786 0.9149408445530416 1.0 0.8645759658113774 0.9708152273052649 0.9127545373146636 0.9140205627861548 0.9553618668971475 0.9742968968384657; 0.7614668093037148 0.9534950355801798 0.761734786836928 0.947585881710825 0.8645759658113774 1.0 0.7571192420493645 0.825113046393563 0.9684360480796209 0.9080207645375546 0.7554275864035706; 0.9784513347398944 0.8888971129999148 0.9902476288270063 0.8288713693959477 0.9708152273052649 0.7571192420493645 0.9999999999999998 0.8882489626913742 0.8215706333107191 0.8906646248524583 0.9831772420358164; 0.8801006621130096 0.8957249145383875 0.8872862085681309 0.8557068486892699 0.9127545373146636 0.825113046393563 0.8882489626913742 1.0 0.8487088583901151 0.8652551113623886 0.8788882102002534; 0.8177765581200408 0.977535983673108 0.8227310530509145 0.9630447457742378 0.9140205627861548 0.9684360480796209 0.8215706333107191 0.8487088583901151 1.0 0.9323664639095011 0.8151501206857463; 0.9266545747580092 0.9760078798048083 0.9079905454268854 0.9418333683726063 0.9553618668971475 0.9080207645375546 0.8906646248524583 0.8652551113623886 0.9323664639095011 1.0 0.9089345178375701; 0.9950881060448081 0.8901094274029715 0.9982814527400847 0.8277230901218361 0.9742968968384657 0.7554275864035706 0.9831772420358164 0.8788882102002534 0.8151501206857463 0.9089345178375701 1.0])
    #     # using LinearAlgebra
    #     # feats = loadfeats("/scratch/sgu3/non-embedding/proteindata/", 18708, "l1gcdm"; numprocs=32)
    #     # println(typeof(feats))
    #     # println(feats[1:5])
    #     # feats = loadfeats("/scratch/sgu3/non-embedding/proteindata/", 18708, "l1node2vec"; numprocs=32)
    #     # println(typeof(feats))
    #     # println(feats[1:5])
    #     # feats = loadfeats("/scratch/sgu3/non-embedding/proteindata/", 18708, "l2node2vec")
    #     # println(typeof(feats))
    #     # println(feats[1:5])
    #     # feats = loadfeats("/scratch/sgu3/non-embedding/proteindata/", 18708, "l1l2node2vecconcat"; numprocs=32)
    #     # println(typeof(feats))
    #     # println(feats[1:5])
    #     # feats = loadfeats("/scratch/sgu3/non-embedding/proteindata/", 18708, "l1l2l2gcdmgdvn2v"; numprocs=32)
    #     # println(typeof(feats))
    #     # println(feats[1:5])
    #     # l2feats = loadfeats("/scratch/sgu3/non-embedding/proteindata/", 18708, "l2gdv")
    #     # println(typeof(l2feats))
    #     # println(l2feats[1:5])
    #
    #     # pool = WorkerPool(collect(1:1))
    #     # testfeats = loadl1gcmtest("/scratch/sgu3/non-embedding/proteindata/l1/", pool, 5)
    #     # l2feats = runorca("/scratch/sgu3/non-embedding/proteindata/l2/l2.sz_hdr.txt")
    #     # using LinearAlgebra
    #     # l2feats = normalize!.(l2feats)
    #     # feats = vcat.(testfeats, l2feats[1:5])
    #     # println(feats)
    end
end
